#include "aClient.h"
#include "config.h"
#include "extern.h"
#include "flags.h"
#include "list.h"
#include "sockhelp.h"

#include <cctype>
#include <fstream.h>

#if defined(HAVE_CRYPT_H)

#include <crypt.h>

#elif defined(HAVE_UNISTD_H)

#include <unistd.h> 

#endif

List<aClient> players;
Monster *monsters[LEVELS][MONSTERS];  	// Monsters per level. Total = MONSTERS * LEVELS

Monster *masters[LEVELS]; 		// A master for each level

// Database functions
int save_gs_dbase();
int load_gs_dbase();

// String functions
#ifndef HAVE_STRTOK
char *strtok(char *str, const char *delim);
#endif

int stricmp(const char *s1, const char *s2);
int strnicmp(const char *s1, const char *s2, size_t len);
// String Functions


/********** Password functions **********/

bool passcmp(char *encrypted, char *plaintext); // Compares an encrypted pass with a plain text one

bool check_password(char *name, char *plaintext); // Finds a password for the given name, and checks it with passcmp against the plaintext password given.
 
/********** Password functions **********/


/********** GameServ Booleans **********/

bool is_playing(char *u); // True if the given nickname in the clients list is playing.
bool is_playing(aClient *user);

bool is_fighting(char *u); // True if the given nick in the clients list is fighting anything.
bool is_fighting(aClient *user);

bool is_alive(char *u); // True if the given nick is playing and is alive
bool is_alive(aClient *user); 

bool player_fight(char *u); // True if the player is fighting another player.
bool player_fight(aClient *user);

bool master_fight(char *u); // True if the player is fighting their master.
bool master_fight(aClient *user);

/********** GameServ Booleans **********/

void display_help(char *u, char *file = NULL);
void display_monster(char *u);
void display_players(char *u);
long int chartoint(char ch);
int isstringnum(char *num);
long int pow (int x, int y);
long int stringtoint(char *number);

char *spaces(int len, char *seperator);
void refresh(Player *p);
void refreshall();
void reset(aClient *ni);
void init_masters();
void init_monsters();
bool load_monsters();
void delete_monsters();
void delete_masters();

void do_admin(char *u);
void do_attack(char *u);
void do_bank(char *u);
void do_fight(char *u);
void do_heal(char *u);
void do_help(char *u);
void do_identify(char *u);
void do_refresh(char *u);
void do_register(char *u);
void do_list(char *u);
void do_master(char *u);
void do_play(char *u);
void do_quitg(char *u);
void do_reset(char *u);
void do_run(char *u);
void do_stats(char *u);
void do_store(char *u);
void see_master(char *u);

void showstats(const char *u, const char *nick);
void showBankBalance(const char *u);

#define WNA 16
char *weapons[WNA] = {  "Fists", "Stick", "Dagger", "Quarterstaff",  "Short Sword", 
			"Long Sword", "Silver Spear", "Battle Axe", "The Ragnarok", 
			"Chain Saw", "Poison Sword",  "Flame Sword", "Earth Hammer", 
			"Light Saber", "Masamune", "Mystical Sword"};

char *armors[WNA] = { "Nothing", "Clothes", "Leather Vest", "Chain Mail", "Plate Armor", 
		      "Full Body Armor", "Magic Mail", "Graphite Suit", "Steel Suit", 
		      "Force Field", "Armor of Light", "Mythril Vest", "DemiGod Armor", 
		      "Hades' Cloak", "Dragon Scales", "Mystical Armor"};

int prices[WNA - 1] = {200, 1000, 3000, 10000, 30000, 100000, 150000, 200000, 400000, 
			1000000, 4000000, 10000000, 40000000, 100000000, 400000000};
int webonus[WNA] = {0, 10, 15, 25, 35, 45, 65, 85, 125, 185, 255, 355, 505, 805, 1205, 1805};
int arbonus[WNA] = {0, 1, 3, 10, 15, 25, 35, 50, 75, 100, 150, 225, 300, 400, 600, 1000};

int hpbonus[11] = {10, 15, 20, 30, 50, 75, 125, 185, 250, 350, 550};
int strbonus[11] = {5, 7, 10, 12, 20, 35, 50, 75, 110, 150, 200};
int defbonus[11] = {2, 3, 5, 10, 15, 22, 35, 60, 80, 120, 150};

void gameserv(char *source, char *buf)
{
    char *cmd;
    cmd = strtok(buf, " ");

    source++; // Get rid of that : at the beginning of a :Nick privmsg Gameserv :text
    cmd++;    // Get rid of that : at the beginning of the :text  (command)

    cout << "Source: " << source << "\ncmd: " << cmd << endl;
    long int mn = midnight() - 12 * 3600; // 12 noon ;)

    if (mn > timestamp)
    {
        refreshall();
        timestamp = mn;
	save_timestamp();
    }

    if (strnicmp(cmd, "\1PING", 6) == 0)
    {
	char *ts;
	ts = strtok(NULL, "\1");
        notice(s_GameServ, source, "\1PING %s\1", ts);
    } else if (stricmp(cmd, "\1VERSION\1") == 0) {
	notice(s_GameServ, source, "\1VERSION %s %s +devel\1", PACKAGE, VERSION);
    } else if (stricmp(cmd, "SEARCH") == 0) {
	cmd = strtok(NULL, " ");

	if (!cmd)
	    notice(s_GameServ, source, "SYNTAX: /msg %S SEARCH FOREST");
	else
	    do_forest(source);

    } else if (stricmp(cmd, "FIGHT") == 0) {
	do_fight(source);
    } else if (stricmp(cmd, "ATTACK") == 0) {
	do_attack(source);
    } else if (stricmp(cmd, "RUN") == 0) {
	do_run(source);
    } else if (stricmp(cmd, "HEAL") == 0) {
	do_heal(source);
    } else if (stricmp(cmd, "MASTER") == 0) {
	do_master(source);
    } else if (stricmp(cmd, "STORE") == 0) {
	do_store(source);
    } else if (stricmp(cmd, "BANK") == 0) {
	do_bank(source);
    } else if (stricmp(cmd, "ADMIN") == 0) {
	do_admin(source);
    } else if (stricmp(cmd, "REFRESH") == 0) {
	do_refresh(source);
    } else if (stricmp(cmd, "PRINT") == 0) {
	cout << "Printing Clients List: " << endl;
	clients.print();
	cout << "\nPrinting Player List: " << endl;
	players.print();
    } else if (stricmp(cmd, "LIST") == 0) {
	do_list(source);
    } else if (stricmp(cmd, "REGISTER") == 0) {
	do_register(source);
    } else if (stricmp(cmd, "IDENTIFY") == 0) {
	do_identify(source);
    } else if (stricmp(cmd, "HELP") == 0) {
	do_help(source);
    } else if (stricmp(cmd, "STATS") == 0) {
	do_stats(source);
    } else if (stricmp(cmd, "SHUTDOWN") == 0) {
	aClient *user;

	if (!(user = find(source)))
	{
	    notice(s_GameServ, source, "Error: aClient not found. Contact a %S admin");
	    cout << "Error: aClient not found: " << source << endl;
	}
	else if (!isAdmin(user))
	{
	    notice(s_GameServ, source, "You must be a %S admin to use this command!");
	}
	else
	{
	    save_gs_dbase();
	    raw("SQUIT %s :leaving", servername);
	}
    } else if (stricmp(cmd, "SAVE") == 0) {
	aClient *user;

	if (!(user = find(source)))
	{
	    notice(s_GameServ, source, "Error: aClient not found. Contact a %S admin");
	    cout << "Error: aClient not found: " << source << endl;
	}
	else if (!isAdmin(user))
	{
	    notice(s_GameServ, source, "You must be a %S admin to use this command!");
	}
	else
        {
	    save_gs_dbase();
        }
    } else if (stricmp(cmd, "LOAD") == 0) {
	aClient *user;

	if (!(user = find(source)))
	{
	    notice(s_GameServ, source, "Error: aClient not found. Contact a %S admin");
	    cout << "Error: aClient not found: " << source << endl;
	}
	else if (!isAdmin(user))
	{
	    notice(s_GameServ, source, "You must be a %S admin to use this command!");
	}
	else
        {
	    char *cmd2 = strtok(NULL, " ");
	    if (!cmd2)
	    {
		notice(s_GameServ, source, "Loading player data from %s", playerdata);
		load_gs_dbase();
	    }
	    else if (stricmp(cmd2, "MONSTERS") == 0)
	    {
		notice(s_GameServ, source, "Loading monster data from %s", monsterdata);
		load_monsters();
	    }
	    else
		display_help(source, cmd);
	}
    } else if (stricmp(cmd, "RAW") == 0) {
	aClient *user;

	if (!(user = find(source)))
	{
	    notice(s_GameServ, source, "Error: aClient not found. Contact a %S admin");
	    cout << "Error: aClient not found: " << source << endl;
	}
	else if (!isAdmin(user))
	{
	    notice(s_GameServ, source, "You must be a %S admin to use this command!");
	}
	else
        {
	    char *rest = strtok(NULL, "");
	    raw("%s", rest);
	}
    } else {
	notice(s_GameServ, source, "Unknown command \002%s\002. Type /msg %S \002HELP\002 to get a list of commands.", cmd);
    } 

   source--;  // Bring the ':' back so we don't leak memory
   cmd--;     // Same thing :)
}

int stricmp(const char *s1, const char *s2)
{
    register int c;

    while ((c = tolower(*s1)) == tolower(*s2)) {
        if (c == 0)
            return 0;
        s1++;
        s2++;
    }
    if (c < tolower(*s2))
        return -1;
    return 1;
}

void showstats(const char *u, const char *nick)
{
    aClient *ni, *sender = find(u);
    char *buf;
    buf = new char[50];
    char *space;


    cout << "\n\nu: " << u << "\nnick: " << nick << endl;
    if (!(ni = findbynick(nick)))
    {
        notice(s_GameServ, u, "%s not found", nick);
    }
    else if (ni->stats)
    {

        notice(s_GameServ, sender->getNick(), "Stats for %s:", ni->stats->name);

        sprintf(buf, "Experience: %ld", ni->stats->exp);
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sLevel: %d",  buf, space,
                 ni->stats->level);
        delete [] space;

        sprintf(buf, "Gold: %ld", ni->stats->gold);
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sGold in Bank: %ld", buf, space, ni->stats->bank);
        delete [] space;

        notice(s_GameServ, sender->getNick(), "Hit Points: %d of %d", ni->stats->hp,
                 ni->stats->maxhp);

        sprintf(buf, "Strength: %d", ni->stats->strength + webonus[ni->stats->weapon]);
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sDefense: %d",
                 buf, space, ni->stats->defense + arbonus[ni->stats->armor]);
        delete [] space;

        sprintf(buf, "Armor: %s", armors[ni->stats->armor]);
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sWeapon: %s", buf, space,
                 weapons[ni->stats->weapon]);
        delete [] space;

        sprintf(buf, "Forest Fights: %d", ni->stats->forest_fights);
        space = spaces(strlen(buf), " ");
        notice(s_GameServ, sender->getNick(), "%s%sPlayer Fights: %d", buf, space, ni->stats->player_fights);
        delete [] space;
    }
    delete [] buf;

}

char *spaces(int len, char *seperator)
{
    char *final;
    final = new char[30];
    int y;
    strcpy(final, seperator);
    for (y = 0; y < 30 - len; y++)
        strcat(final, seperator);
    return final;
}

void raw(const char *fmt, ...)
{
    va_list args;
    char *input;
    const char *t = fmt;
    input = new char[1024];
    va_start(args, fmt);
    memset(input, 0, sizeof(input)); // Initialize to NULL
    for (; *t; t++)
    {
	if (*t == '%')
	{
	    switch(*++t) {
		case 'd': sprintf(input, "%s%d", input, va_arg(args, int)); break;
		case 's': sprintf(input, "%s%s", input, va_arg(args, char *)); break;
		case 'S': sprintf(input, "%s%s", input, s_GameServ); break;
		case 'l':
		   if (*++t == 'd')
			  sprintf(input, "%s%ld", input, va_arg(args, long int)); break;
	    }
	}
	else
	{
	    sprintf(input, "%s%c", input, *t);
	}

    }
    sprintf(input, "%s%s", input, "\r\n");
    cout << "input: " << input << flush;
    sock_puts(sock, input);
    delete [] input;
    va_end(args);
}
/* Send a NOTICE from the given source to the given nick. */

void notice(const char *source, const char *dest, const char *fmt, ...)
{
    if (fmt[0] == '\0')
	return;

    va_list args;
    char *input;
    const char *t = fmt;
    input = new char[1024];
    va_start(args, fmt);
    if (dest[0] == ':')
    {
    	dest++;
	sprintf(input, ":%s NOTICE %s :", source, dest);
	dest--;
    }
    else
	sprintf(input, ":%s NOTICE %s :", source, dest);

    for (; *t; t++)
    {
	if (*t == '%')
	{
	    switch(*++t) {
		case 'd': sprintf(input, "%s%d", input, va_arg(args, int)); break; 
		case 's': sprintf(input, "%s%s", input, va_arg(args, char *)); break;
		case 'S': sprintf(input, "%s%s", input, s_GameServ); break;
		case 'l':
		   if (*++t == 'd')
			  sprintf(input, "%s%ld", input, va_arg(args, long int)); break;
	    }
	}
	else
	{
	    sprintf(input, "%s%c", input, *t);
	}

    }
    sprintf(input, "%s%s", input, "\r\n");
    cout << "input: " << input << flush;
    sock_puts(sock, input);
    delete [] input;
va_end(args);
}


int strnicmp(const char *s1, const char *s2, size_t len)
{
    register int c;

    if (!len)
        return 0;
    while ((c = tolower(*s1)) == tolower(*s2) && len > 0) {
        if (c == 0 || --len == 0)
            return 0;
        s1++;
        s2++;
    }
    if (c < tolower(*s2))
        return -1;
    return 1;
}

#ifndef HAVE_STRTOK
char *strtok(char *str, const char *delim)
{
    static char *current = NULL;
    char *ret;

    if (str)
        current = str;
    if (!current)
        return NULL;
    current += strspn(current, delim);
    ret = *current ? current : NULL;
    current += strcspn(current, delim);
    if (!*current)
        current = NULL;
    else
        *current++ = 0;
    return ret;
}
#endif

void do_list(char *u)
{
    ListNode<aClient> *temp;
    temp = players.First();
    if (!players.isEmpty())
    {
	notice(s_GameServ, u, "People Playing:");
	while(temp)
	{
	    notice(s_GameServ, u, "IRC: %s     Game: %s", temp->getData()->getNick(), temp->getData()->stats->name);
	    temp = temp->Next();
	}
	notice(s_GameServ, u, "End of List");
    }
    else
	notice(s_GameServ, u, "No one is playing");
}

void do_register(char *u)
{
    char *password;
    aClient *user;
    password = strtok(NULL, " ");

    static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
    static char salt[3];
    
    salt[0] = saltChars[rand() % strlen(saltChars)];
    salt[1] = saltChars[rand() % strlen(saltChars)];
    salt[3] = '\0';

    if (!password)
    {
	notice(s_GameServ, u, "SYNTAX: /msg %S REGISTER PASSWORD");
    }
    else if ((user = find(u)))
    {
        if (!user->stats)
        {
	    user->stats = new Player(user);
	    user->stats->user = user; // Set the backwards pointer
	    strcpy(user->stats->password, crypt(password, salt));
	    players.insertAtBack(user);
	    notice(s_GameServ, u, "Player %s registered with password %s.", user->stats->name, password);
	    notice(s_GameServ, u, "Write this password down. If you lose it, there is no way to retrieve it!");
	}
	else
	{
	    notice(s_GameServ, u, "Already registered. Contact a %S admin for help.");
	}
    }
}

void do_identify(char *u)
{
    char *password, *name;
    aClient *user, *p;
    name = strtok(NULL, " ");
    password = strtok(NULL, " ");

    if (!password || !name)
    {
	notice(s_GameServ, u, "SYNTAX: /msg %S IDENTIFY NAME PASSWORD");
    }
    else if (!(p = findplayer(name)) || !p->stats)
	    notice(s_GameServ, u, "Player %s not found", name);
    else if (!check_password(name, password))
    {
	    notice(s_GameServ, u, "Password incorrect");
    }
    else if ((user = find(u)))
    {
        if (!user->stats)
        {
	    ListNode<aClient> *temp;
	    temp = players.Find(p);
	    if (!temp)
	    {
		notice(s_GameServ, u, "Fatal error. Contact %S Admin. Buf: %s", 
			strtok(NULL, ""));
		return;
	    }
	    user->stats = new Player(p->stats->name);
	    cout << "Setting data for identified" << endl;
	    user->stats->setData(p->stats);
	    cout << "Player Identified" << endl << flush;

	    temp->setPtr(user);

	    notice(s_GameServ, u, "Password Accepted. Identified.");	    

	}
	else
	{
	    notice(s_GameServ, u, "Already identified. Contact a %S admin for help.");
	}
    }
}

void do_stats(char *u)
{
    char *nick;
    aClient *source;

    nick = strtok(NULL, " ");
    source = find(u);

    if (!nick)
	showstats(u, source->getNick());
    else
	showstats(u, nick);
}
void init_masters()
{
    delete_masters();

    for (int x = 0; x < LEVELS; x++)
	masters[x] = new Monster;

    strcpy(masters[0]->name, "Old Bones");
    strcpy(masters[0]->weapon, "Dull Sword Cane");
    masters[0]->strength = 15;
    masters[0]->gold = 0;
    masters[0]->exp = 0;
    masters[0]->maxhp = 30;
    masters[0]->hp = 30;
    strcpy(masters[0]->death, "You have done well my student, but the road is long. Use your new strength with humility and honor as you progress in levels!");

    strcpy(masters[1]->name, "Master Chang");
    strcpy(masters[1]->weapon, "Nanchaku");
    masters[1]->strength = 30;
    masters[1]->gold = 0;
    masters[1]->exp = 0;
    masters[1]->maxhp = 40;
    masters[1]->hp = 40;
    strcpy(masters[1]->death, "You try to make out what Master Chang is saying, but the only thing you catch is something about a grasshopper.");

    strcpy(masters[2]->name, "Chuck Norris");
    strcpy(masters[2]->weapon, "Ranger Kick");
    masters[2]->strength = 85;
    masters[2]->gold = 0;
    masters[2]->exp = 0;
    masters[2]->maxhp = 70;
    masters[2]->hp = 70;
    strcpy(masters[2]->death, "Be strong, and keep your goals in site. Drink milk, and don't do drugs. One day you may be fighting next to me as a Texas Ranger YEEHAW!");


    strcpy(masters[3]->name, "Mr. Miagi");
    strcpy(masters[3]->weapon, "Petrified Bonsai");
    masters[3]->strength = 100;
    masters[3]->gold = 0;
    masters[3]->exp = 0;
    masters[3]->maxhp = 120;
    masters[3]->hp = 120;
    strcpy(masters[3]->death, "Skill comes from repeating the correct but seemingly mundane actions. Wax ON, wax OFF!");

    strcpy(masters[4]->name, "Jackie Chan");
    strcpy(masters[4]->weapon, "Kung Fu Kick");
    masters[4]->strength = 125;
    masters[4]->gold = 0;
    masters[4]->exp = 0;
    masters[4]->maxhp = 200;
    masters[4]->hp = 200;
    strcpy(masters[4]->death, "I like to let people talk who like to talk... it's easier to find out how full of it they really are!");

    strcpy(masters[5]->name, "Jet Li");
    strcpy(masters[5]->weapon, "Motorcycle");
    masters[5]->strength = 150;
    masters[5]->gold = 0;
    masters[5]->exp = 0;
    masters[5]->maxhp = 400;
    masters[5]->hp = 400;
    strcpy(masters[5]->death, "Failure is a fuel for excuses. It's the doing the do, that makes the making.");


    strcpy(masters[6]->name, "Muhammad Ali");
    strcpy(masters[6]->weapon, "Quick Jab");
    masters[6]->strength = 175;
    masters[6]->gold = 0;
    masters[6]->exp = 0;
    masters[6]->maxhp = 600;
    masters[6]->hp = 600;
    strcpy(masters[6]->death, "It's just a job. Grass grows, birds fly, waves pound the sand. I beat people up.");

    strcpy(masters[7]->name, "Li Mu Bai");
    strcpy(masters[7]->weapon, "Green Destiny");
    masters[7]->strength = 200;
    masters[7]->gold = 0;
    masters[7]->exp = 0;
    masters[7]->maxhp = 800;
    masters[7]->hp = 800;
    strcpy(masters[7]->death, "No growth without resistance.  No action without reaction.  No desire without restraint.");


    strcpy(masters[8]->name, "Jimmy Wang Yu");
    strcpy(masters[8]->weapon, "Flying Guillotine");
    masters[8]->strength = 275;
    masters[8]->gold = 0;
    masters[8]->exp = 0;
    masters[8]->maxhp = 1200;
    masters[8]->hp = 1200;
    strcpy(masters[8]->death, "You have beaten the one armed boxer. Proceed with caution!");

    strcpy(masters[9]->name, "Wong Fei Hung");
    strcpy(masters[9]->weapon, "Drunken Boxing");
    masters[9]->strength = 350;
    masters[9]->gold = 0;
    masters[9]->exp = 0;
    masters[9]->maxhp = 1800;
    masters[9]->hp = 1800;
    strcpy(masters[9]->death, "Hiccup! Monkey drinks master's wine!");

    strcpy(masters[10]->name, "Bruce Lee");
    strcpy(masters[10]->weapon, "Fists of fury");
    masters[10]->strength = 575;
    masters[10]->gold = 0;
    masters[10]->exp = 0;
    masters[10]->maxhp = 2500;
    masters[10]->hp = 2500;
    strcpy(masters[10]->death, "You must learn to concentrate. It is like a finger pointing away to the moon... DONT concentrate on the finger, or you will miss all the heavenly glory.");
}

void init_monsters()
{
    delete_monsters();
    for (int x = 0; x < LEVELS; x++)
	for (int y = 0; y < MONSTERS; y++)
	    monsters[x][y] = new Monster();

    // Hard coded for now - Kain
/*

    strcpy(monsters[0][0]->name, "Slime");
    strcpy(monsters[0][0]->weapon, "Acid Goo");
    monsters[0][0]->strength = 6;
    monsters[0][0]->gold = 50;
    monsters[0][0]->exp = 3;
    monsters[0][0]->maxhp = 9;
    strcpy(monsters[0][0]->death, "The slime oozes into nothing... you clean the acid goo off of your weapon");

    strcpy(monsters[0][1]->name, "Ghost");
    strcpy(monsters[0][1]->weapon, "Cold Breath");
    monsters[0][1]->strength = 8;
    monsters[0][1]->gold = 100;
    monsters[0][1]->exp = 10;
    monsters[0][1]->maxhp = 10;
    strcpy(monsters[0][1]->death, "You feel a chill as the spirit leaves the realm.");

    strcpy(monsters[0][2]->name, "Ugly Rodent");
    strcpy(monsters[0][2]->weapon, "Sharp Teeth");
    monsters[0][2]->strength = 9;
    monsters[0][2]->gold = 75;
    monsters[0][2]->exp = 8;
    monsters[0][2]->maxhp = 13;
    strcpy(monsters[0][2]->death, "You stomp on the Ugly Rodent's remains for a finishing blow.");

    strcpy(monsters[0][3]->name, "Whart Hog");
    strcpy(monsters[0][3]->weapon, "Tusks");
    monsters[0][3]->strength = 10;
    monsters[0][3]->gold = 80;
    monsters[0][3]->exp = 6;
    monsters[0][3]->maxhp = 10;
    strcpy(monsters[0][3]->death, "You cook and eat the hog for good measure!");

    strcpy(monsters[0][4]->name, "Pesky Kid");
    strcpy(monsters[0][4]->weapon, "Slingshot");
    monsters[0][4]->strength = 8;
    monsters[0][4]->gold = 30;
    monsters[0][4]->exp = 4;
    monsters[0][4]->maxhp = 6;
    strcpy(monsters[0][4]->death, "You take his slingshot and snap the band, sending the kid crying home to mom!");

    strcpy(monsters[0][5]->name, "Playground Bully");
    strcpy(monsters[0][5]->weapon, "Painful Noogie");
    monsters[0][5]->strength = 11;
    monsters[0][5]->gold = 44;
    monsters[0][5]->exp = 6;
    monsters[0][5]->maxhp = 10;
    strcpy(monsters[0][5]->death, "You give him an indian burn, and punt him across the schoolyard!");

    strcpy(monsters[0][6]->name, "Small Imp");
    strcpy(monsters[0][6]->weapon, "Dagger");
    monsters[0][6]->strength = 6;
    monsters[0][6]->gold = 64;
    monsters[0][6]->exp = 10;
    monsters[0][6]->maxhp = 10;
    strcpy(monsters[0][6]->death, "You can't help but laugh as he stumbles and falls onto his own dagger!");

    strcpy(monsters[0][7]->name, "Little Monkey");
    strcpy(monsters[0][7]->weapon, "Monkey Wrench");
    monsters[0][7]->strength = 6;
    monsters[0][7]->gold = 53;
    monsters[0][7]->exp = 9;
    monsters[0][7]->maxhp = 9;
    strcpy(monsters[0][7]->death, "You want to cook it, but you just can't think of eating something that looks so human!");

    strcpy(monsters[0][8]->name, "Grub Worm");
    strcpy(monsters[0][8]->weapon, "Minor Nudge");
    monsters[0][8]->strength = 2;
    monsters[0][8]->gold = 10;
    monsters[0][8]->exp = 3;
    monsters[0][8]->maxhp = 3;
    strcpy(monsters[0][8]->death, "You decide to save the poor little fella for your next fishing trip.");

    strcpy(monsters[0][9]->name, "Drakee");
    strcpy(monsters[0][9]->weapon, "Tail Slap");
    monsters[0][9]->strength = 5;
    monsters[0][9]->gold = 22;
    monsters[0][9]->exp = 7;
    monsters[0][9]->maxhp = 5;
    strcpy(monsters[0][9]->death, "You pull the little Drakee by its tale and slam it down on a dry stump!");

    strcpy(monsters[0][10]->name, "Fat Slob");
    strcpy(monsters[0][10]->weapon, "Smelly Breath");
    monsters[0][10]->strength = 6;
    monsters[0][10]->gold = 40;
    monsters[0][10]->exp = 10;
    monsters[0][10]->maxhp = 7;
    strcpy(monsters[0][10]->death, "You kick his stomach for fun, and are thrown back by the spring of it all!");

    strcpy(monsters[0][11]->name, "Lost Warrior");
    strcpy(monsters[0][11]->weapon, "Long Sword");
    monsters[0][11]->strength = 10;
    monsters[0][11]->gold = 250;
    monsters[0][11]->exp = 19;
    monsters[0][11]->maxhp = 15;
    strcpy(monsters[0][11]->death, "You give him a proper burial in respect for the dead warrior.");

    strcpy(monsters[1][0]->name, "Lost Warrior's Cousin Larry");
    strcpy(monsters[1][0]->weapon, "Wood Axe");
    monsters[1][0]->strength = 19;
    monsters[1][0]->gold = 134;
    monsters[1][0]->exp = 24;
    monsters[1][0]->maxhp = 30;
    strcpy(monsters[1][0]->death, "He was pretty pissed you killed his cousin, but he seems to have suffered the same fate!");

    strcpy(monsters[1][1]->name, "Sandman");
    strcpy(monsters[1][1]->weapon, "Sleeping Dust");
    monsters[1][1]->strength = 25;
    monsters[1][1]->gold = 80;
    monsters[1][1]->exp = 6;
    monsters[1][1]->maxhp = 27;
    strcpy(monsters[1][1]->death, "You put the sandman to his final sleep.");

    strcpy(monsters[1][2]->name, "Dirty Prostitute");
    strcpy(monsters[1][2]->weapon, "Stiletto Heel");
    monsters[1][2]->strength = 21;
    monsters[1][2]->gold = 160;
    monsters[1][2]->exp = 12;
    monsters[1][2]->maxhp = 25;
    strcpy(monsters[1][2]->death, "Your pimp hand is strong!");

    strcpy(monsters[1][3]->name, "Goblin Gardener");
    strcpy(monsters[1][3]->weapon, "Garden Spade");
    monsters[1][3]->strength = 18;
    monsters[1][3]->gold = 130;
    monsters[1][3]->exp = 8;
    monsters[1][3]->maxhp = 20;
    strcpy(monsters[1][3]->death, "You trample on his garden after slaying him... that felt good!");

    strcpy(monsters[1][4]->name, "Evil Elf");
    strcpy(monsters[1][4]->weapon, "Dark Bow");
    monsters[1][4]->strength = 23;
    monsters[1][4]->gold = 136;
    monsters[1][4]->exp = 13;
    monsters[1][4]->maxhp = 24;
    strcpy(monsters[1][4]->death, "Elves are usually nice you thought... hmm.");

    strcpy(monsters[1][5]->name, "Viking Warrior");
    strcpy(monsters[1][5]->weapon, "Broad Sword");
    monsters[1][5]->strength = 21;
    monsters[1][5]->gold = 330;
    monsters[1][5]->exp = 20;
    monsters[1][5]->maxhp = 18;
    strcpy(monsters[1][5]->death, "You heard vikings were big, but not THAT big you thought.");

    strcpy(monsters[1][6]->name, "Wicked Witch");
    strcpy(monsters[1][6]->weapon, "Cackling Laugh");
    monsters[1][6]->strength = 20;
    monsters[1][6]->gold = 130;
    monsters[1][6]->exp = 20;
    monsters[1][6]->maxhp = 26;
    strcpy(monsters[1][6]->death, "Just for kicks, you splash some water on her and watch her melt.");

    strcpy(monsters[1][7]->name, "Vampire Bat");
    strcpy(monsters[1][7]->weapon, "Blood Sucking Fangs");
    monsters[1][7]->strength = 18;
    monsters[1][7]->gold = 125;
    monsters[1][7]->exp = 21;
    monsters[1][7]->maxhp = 29;
    strcpy(monsters[1][7]->death, "You fry up the bat and eat it... needs garlic.");

    strcpy(monsters[1][8]->name, "Thorn Bush");
    strcpy(monsters[1][8]->weapon, "101 Thorns");
    monsters[1][8]->strength = 16;
    monsters[1][8]->gold = 94;
    monsters[1][8]->exp = 15;
    monsters[1][8]->maxhp = 25;
    strcpy(monsters[1][8]->death, "You set the bush ablaze and roast some marshmallows.");

    strcpy(monsters[1][9]->name, "Barbarian");
    strcpy(monsters[1][9]->weapon, "Heavy Sword");
    monsters[1][9]->strength = 29;
    monsters[1][9]->gold = 250;
    monsters[1][9]->exp = 25;
    monsters[1][9]->maxhp = 30;
    strcpy(monsters[1][9]->death, "You listen to him moan as he falls over dead.");

    strcpy(monsters[1][10]->name, "Crypt Rat");
    strcpy(monsters[1][10]->weapon, "Stinging Bite");
    monsters[1][10]->strength = 25;
    monsters[1][10]->gold = 119;
    monsters[1][10]->exp = 20;
    monsters[1][10]->maxhp = 26;
    strcpy(monsters[1][10]->death, "You squash the little rodent for fear that it might not be dead.");

    strcpy(monsters[1][11]->name, "Small Orc");
    strcpy(monsters[1][11]->weapon, "blade");
    monsters[1][11]->strength = 28;
    monsters[1][11]->gold = 300;
    monsters[1][11]->exp = 30;
    monsters[1][11]->maxhp = 36;
    strcpy(monsters[1][11]->death, "It's an ugly one, and it would've grown up to be a terror...");

    strcpy(monsters[2][0]->name, "Teferi");
    strcpy(monsters[2][0]->weapon, "Puzzle Box");
    monsters[2][0]->strength = 29;
    monsters[2][0]->gold = 380;
    monsters[2][0]->exp = 18;
    monsters[2][0]->maxhp = 29;
    strcpy(monsters[2][0]->death, "It was a puzzling experience.");

    strcpy(monsters[2][1]->name, "Spineless Thug");
    strcpy(monsters[2][1]->weapon, "Spiked Bat");
    monsters[2][1]->strength = 37;
    monsters[2][1]->gold = 384;
    monsters[2][1]->exp = 27;
    monsters[2][1]->maxhp = 32;
    strcpy(monsters[2][1]->death, "See you at the crossroads!");

    strcpy(monsters[2][2]->name, "Pyromaniac");
    strcpy(monsters[2][2]->weapon, "Flame Thrower");
    monsters[2][2]->strength = 29;
    monsters[2][2]->gold = 563;
    monsters[2][2]->exp = 22;
    monsters[2][2]->maxhp = 45;
    strcpy(monsters[2][2]->death, "He chants FIRE FIRE as he falls to the ground... a burning heap of flesh.");

    strcpy(monsters[2][3]->name, "Evil Enchantress");
    strcpy(monsters[2][3]->weapon, "Deadly Spell");
    monsters[2][3]->strength = 50;
    monsters[2][3]->gold = 830;
    monsters[2][3]->exp = 35;
    monsters[2][3]->maxhp = 35;
    strcpy(monsters[2][3]->death, "She looked just about as good as she fought.");

    strcpy(monsters[2][4]->name, "Killer Leprechaun");
    strcpy(monsters[2][4]->weapon, "Gold Rush");
    monsters[2][4]->strength = 35;
    monsters[2][4]->gold = 1300;
    monsters[2][4]->exp = 30;
    monsters[2][4]->maxhp = 37;
    strcpy(monsters[2][4]->death, "You steal his pot of gold... that's a lot of money!");

    strcpy(monsters[2][5]->name, "Avalanche Rider");
    strcpy(monsters[2][5]->weapon, "Huge Snowball");
    monsters[2][5]->strength = 32;
    monsters[2][5]->gold = 700;
    monsters[2][5]->exp = 32;
    monsters[2][5]->maxhp = 38;
    strcpy(monsters[2][5]->death, "You take his snowboard and snap it in two!");

    strcpy(monsters[2][6]->name, "Blundering Idiot");
    strcpy(monsters[2][6]->weapon, "Stupidity");
    monsters[2][6]->strength = 14;
    monsters[2][6]->gold = 700;
    monsters[2][6]->exp = 20;
    monsters[2][6]->maxhp = 29;
    strcpy(monsters[2][6]->death, "Now there's one person you don't feel sorry for killing!");

    strcpy(monsters[2][7]->name, "Militant Anarchist");
    strcpy(monsters[2][7]->weapon, "Molotov Cocktail");
    monsters[2][7]->strength = 33;
    monsters[2][7]->gold = 245;
    monsters[2][7]->exp = 45;
    monsters[2][7]->maxhp = 32;
    strcpy(monsters[2][7]->death, "Order has been restored for now...");

    strcpy(monsters[2][8]->name, "Scathe Zombies");
    strcpy(monsters[2][8]->weapon, "Death Grip");
    monsters[2][8]->strength = 38;
    monsters[2][8]->gold = 763;
    monsters[2][8]->exp = 15;
    monsters[2][8]->maxhp = 45;
    strcpy(monsters[2][8]->death, "That was perhaps the scariest experience of your life.");

    strcpy(monsters[2][9]->name, "Spitting Llama");
    strcpy(monsters[2][9]->weapon, "Spit Spray");
    monsters[2][9]->strength = 48;
    monsters[2][9]->gold = 638;
    monsters[2][9]->exp = 28;
    monsters[2][9]->maxhp = 34;
    strcpy(monsters[2][9]->death, "You wipe the spit off your face and fling it back at the Llama.");

    strcpy(monsters[2][10]->name, "Juggalo");
    strcpy(monsters[2][10]->weapon, "Clown Axe");
    monsters[2][10]->strength = 60;
    monsters[2][10]->gold = 650;
    monsters[2][10]->exp = 30;
    monsters[2][10]->maxhp = 29;
    strcpy(monsters[2][10]->death, "What is a Juggalo? I don't know!");

    strcpy(monsters[2][11]->name, "The Boogie Man");
    strcpy(monsters[2][11]->weapon, "Striking Fear");
    monsters[2][11]->strength = 46;
    monsters[2][11]->gold = 600;
    monsters[2][11]->exp = 35;
    monsters[2][11]->maxhp = 27;
    strcpy(monsters[2][11]->death, "He's scared you for the very last time!");

    strcpy(monsters[3][0]->name, "Living Fire");
    strcpy(monsters[3][0]->weapon, "Scorching Wind");
    monsters[3][0]->strength = 55;
    monsters[3][0]->gold = 1100;
    monsters[3][0]->exp = 36;
    monsters[3][0]->maxhp = 55;
    strcpy(monsters[3][0]->death, "You extinguish the Living Flame once and for all!");

    strcpy(monsters[3][1]->name, "Raging Orc");
    strcpy(monsters[3][1]->weapon, "Orcish Artillary");
    monsters[3][1]->strength = 89;
    monsters[3][1]->gold = 900;
    monsters[3][1]->exp = 25;
    monsters[3][1]->maxhp = 50;
    strcpy(monsters[3][1]->death, "This orc was a bit tougher than you remembered!");

    strcpy(monsters[3][2]->name, "Huge Tarantula");
    strcpy(monsters[3][2]->weapon, "Tangling Web");
    monsters[3][2]->strength = 59;
    monsters[3][2]->gold = 1000;
    monsters[3][2]->exp = 35;
    monsters[3][2]->maxhp = 60;
    strcpy(monsters[3][2]->death, "You're glad you overcame your arachniphobia so soon!");

    strcpy(monsters[3][3]->name, "Rabid Wolf");
    strcpy(monsters[3][3]->weapon, "Cujo Bite");
    monsters[3][3]->strength = 40;
    monsters[3][3]->gold = 1200;
    monsters[3][3]->exp = 47;
    monsters[3][3]->maxhp = 76;
    strcpy(monsters[3][3]->death, "The mutt falls over dead as white foam drips from its deadly canines...");

    strcpy(monsters[3][4]->name, "Goblin Fighter");
    strcpy(monsters[3][4]->weapon, "Morning Star");
    monsters[3][4]->strength = 38;
    monsters[3][4]->gold = 700;
    monsters[3][4]->exp = 30;
    monsters[3][4]->maxhp = 75;
    strcpy(monsters[3][4]->death, "He almost caught you with his chain mace, but you sliced off his head.");

    strcpy(monsters[3][5]->name, "Grizzly Bear");
    strcpy(monsters[3][5]->weapon, "Razor Claws");
    monsters[3][5]->strength = 68;
    monsters[3][5]->gold = 1747;
    monsters[3][5]->exp = 81;
    monsters[3][5]->maxhp = 51;
    strcpy(monsters[3][5]->death, "It almost got you this time... better be careful");

    strcpy(monsters[3][6]->name, "Skeleton Man");
    strcpy(monsters[3][6]->weapon, "Leg Bone");
    monsters[3][6]->strength = 70;
    monsters[3][6]->gold = 597;
    monsters[3][6]->exp = 57;
    monsters[3][6]->maxhp = 60;
    strcpy(monsters[3][6]->death, "As a finisher, you wind up with the broad side of your weapon and hit his skull off for a home run!");

    strcpy(monsters[3][7]->name, "Young Werewolf");
    strcpy(monsters[3][7]->weapon, "Howling Bites");
    monsters[3][7]->strength = 75;
    monsters[3][7]->gold = 1742;
    monsters[3][7]->exp = 65;
    monsters[3][7]->maxhp = 42;
    strcpy(monsters[3][7]->death, "You scatter the wolf's body parts in hopes he will stay dead!");

    strcpy(monsters[3][8]->name, "Dark Infantry");
    strcpy(monsters[3][8]->weapon, "Flesh Reaper");
    monsters[3][8]->strength = 69;
    monsters[3][8]->gold = 870;
    monsters[3][8]->exp = 43;
    monsters[3][8]->maxhp = 65;
    strcpy(monsters[3][8]->death, "Light has prevailed this time... but it's only so long before you meet again.");

    strcpy(monsters[3][9]->name, "Erie Spirit");
    strcpy(monsters[3][9]->weapon, "Deadly Grin");
    monsters[3][9]->strength = 63;
    monsters[3][9]->gold = 1300;
    monsters[3][9]->exp = 32;
    monsters[3][9]->maxhp = 50;
    strcpy(monsters[3][9]->death, "His cousin the ghost was a little bit easier.");

    strcpy(monsters[3][10]->name, "Gollum");
    strcpy(monsters[3][10]->weapon, "Precious Treasure");
    monsters[3][10]->strength = 66;
    monsters[3][10]->gold = 1492;
    monsters[3][10]->exp = 73;
    monsters[3][10]->maxhp = 54;
    strcpy(monsters[3][10]->death, "Gollum screams out \"MY PRECIOUS\" as his small body falls limp from your blow.");

    strcpy(monsters[3][11]->name, "Rock Fighter");
    strcpy(monsters[3][11]->weapon, "Small Boulders");
    monsters[3][11]->strength = 87;
    monsters[3][11]->gold = 1742;
    monsters[3][11]->exp = 99;
    monsters[3][11]->maxhp = 65;
    strcpy(monsters[3][11]->death, "You dodge his last rock, and counter with a low blow, cutting off his legs.");


    strcpy(monsters[4][0]->name, "Giant Sphinx");
    strcpy(monsters[4][0]->weapon, "Ancient Curse");
    monsters[4][0]->strength = 120;
    monsters[4][0]->gold = 1000;
    monsters[4][0]->exp = 100;
    monsters[4][0]->maxhp = 80;
    strcpy(monsters[4][0]->death, "You look in awe at the great wonder, collapsed at your feet!");

    strcpy(monsters[4][1]->name, "Giant Ogre");
    strcpy(monsters[4][1]->weapon, "Big Log");
    monsters[4][1]->strength = 130;
    monsters[4][1]->gold = 857;
    monsters[4][1]->exp = 175;
    monsters[4][1]->maxhp = 100;
    strcpy(monsters[4][1]->death, "Your wits outmatched the ogre's brawn... big dumb thing.");

    strcpy(monsters[4][2]->name, "Massive Cockroach");
    strcpy(monsters[4][2]->weapon, "Piercing Hiss");
    monsters[4][2]->strength = 125;
    monsters[4][2]->gold = 700;
    monsters[4][2]->exp = 150;
    monsters[4][2]->maxhp = 112;
    strcpy(monsters[4][2]->death, "Where's the exterminator when you need one?");

    strcpy(monsters[4][3]->name, "Big Venomous Snake");
    strcpy(monsters[4][3]->weapon, "Poison Fangs");
    monsters[4][3]->strength = 140;
    monsters[4][3]->gold = 900;
    monsters[4][3]->exp = 175;
    monsters[4][3]->maxhp = 126;
    strcpy(monsters[4][3]->death, "After killing this beast you check for puncture marks... you find none, luckily.");

    strcpy(monsters[4][4]->name, "Lizard Man");
    strcpy(monsters[4][4]->weapon, "Deadly Jaws");
    monsters[4][4]->strength = 145;
    monsters[4][4]->gold = 1250;
    monsters[4][4]->exp = 175;
    monsters[4][4]->maxhp = 150;
    strcpy(monsters[4][4]->death, "His scales made for tough armor, and his jaws for a tougher opponent!");

    strcpy(monsters[4][5]->name, "Face Dancer");
    strcpy(monsters[4][5]->weapon, "Illusion Scyth");
    monsters[4][5]->strength = 138;
    monsters[4][5]->gold = 1603;
    monsters[4][5]->exp = 198;
    monsters[4][5]->maxhp = 173;
    strcpy(monsters[4][5]->death, "His carcus takes the shape of many things before it dies. His true form is so repulsive, you know why he changed faces so much!");

    strcpy(monsters[4][6]->name, "Darklord Longbow Archer");
    strcpy(monsters[4][6]->weapon, "Deadly Bow and Arrows");
    monsters[4][6]->strength = 145;
    monsters[4][6]->gold = 1569;
    monsters[4][6]->exp = 243;
    monsters[4][6]->maxhp = 170;
    strcpy(monsters[4][6]->death, "Your face turns white with horror after you realize you just met the devil's protector!");

    strcpy(monsters[4][7]->name, "Hell's Paladin");
    strcpy(monsters[4][7]->weapon, "Sword of Hellfire");
    monsters[4][7]->strength = 200;
    monsters[4][7]->gold = 2191;
    monsters[4][7]->exp = 254;
    monsters[4][7]->maxhp = 175;
    strcpy(monsters[4][7]->death, "This is starting to get tough you think. Do you really want to go to level 6?");

    strcpy(monsters[4][8]->name, "The Unknown Soldier");
    strcpy(monsters[4][8]->weapon, "Soul Torture");
    monsters[4][8]->strength = 175;
    monsters[4][8]->gold = 1890;
    monsters[4][8]->exp = 200;
    monsters[4][8]->maxhp = 180;
    strcpy(monsters[4][8]->death, "Who was that? Where was he from? And what was that weapon??");

    strcpy(monsters[4][9]->name, "Undead Cult Leader");
    strcpy(monsters[4][9]->weapon, "Lance of Deceit");
    monsters[4][9]->strength = 180;
    monsters[4][9]->gold = 1792;
    monsters[4][9]->exp = 195;
    monsters[4][9]->maxhp = 190;
    strcpy(monsters[4][9]->death, "His words fall on deaf ears... this is one cult you will NOT be part of!");

    strcpy(monsters[4][10]->name, "Water Serpent");
    strcpy(monsters[4][10]->weapon, "Forked Tongue");
    monsters[4][10]->strength = 150;
    monsters[4][10]->gold = 1500;
    monsters[4][10]->exp = 176;
    monsters[4][10]->maxhp = 220;
    strcpy(monsters[4][10]->death, "The serpent squeals as you cut off its head!");

    strcpy(monsters[4][11]->name, "Silverback Gorilla");
    strcpy(monsters[4][11]->weapon, "Deadly Banana Peel");
    monsters[4][11]->strength = 160;
    monsters[4][11]->gold = 1300;
    monsters[4][11]->exp = 150;
    monsters[4][11]->maxhp = 178;
    strcpy(monsters[4][11]->death, "Was that gorilla or guerilla?");

    strcpy(	monsters[5][0]->name, "");
    strcpy(	monsters[5][0]->weapon, "");
		monsters[5][0]->strength = 200;
		monsters[5][0]->gold = 2500;
		monsters[5][0]->exp = 200;
		monsters[5][0]->maxhp = 200;
    strcpy(	monsters[5][0]->death, "");

    strcpy(	monsters[5][1]->name, "");
    strcpy(	monsters[5][1]->weapon, "");
		monsters[5][1]->strength = 230;
		monsters[5][1]->gold = 3215;
		monsters[5][1]->exp = 230;
		monsters[5][1]->maxhp = 210;
    strcpy(	monsters[5][1]->death, "");

    strcpy(	monsters[5][2]->name, "");
    strcpy(	monsters[5][2]->weapon, "");
		monsters[5][2]->strength = 215;
		monsters[5][2]->gold = 2750;
		monsters[5][2]->exp = 245;
		monsters[5][2]->maxhp = 214;
    strcpy(	monsters[5][2]->death, "");

    strcpy(	monsters[5][3]->name, "Cheap Bastard");
    strcpy(	monsters[5][3]->weapon, "Sack of Pennies");
		monsters[5][3]->strength = 245;
		monsters[5][3]->gold = 1000;
		monsters[5][3]->exp = 195;
		monsters[5][3]->maxhp = 200;
    strcpy(	monsters[5][3]->death, "You hear him scream, \"MY COINS!!\" as you run off with his sack.");

    strcpy(	monsters[5][4]->name, "");
    strcpy(	monsters[5][4]->weapon, "");
		monsters[5][4]->strength = 300;
		monsters[5][4]->gold = 4500;
		monsters[5][4]->exp = 375;
		monsters[5][4]->maxhp = 221;
    strcpy(	monsters[5][4]->death, "");

    strcpy(	monsters[5][5]->name, "");
    strcpy(	monsters[5][5]->weapon, "");
		monsters[5][5]->strength = 1;
		monsters[5][5]->gold = 1;
		monsters[5][5]->exp = 1;
		monsters[5][5]->maxhp = 1;
    strcpy(	monsters[5][5]->death, "");

    strcpy(	monsters[5][6]->name, "");
    strcpy(	monsters[5][6]->weapon, "");
		monsters[5][6]->strength = 1;
		monsters[5][6]->gold = 1;
		monsters[5][6]->exp = 1;
		monsters[5][6]->maxhp = 1;
    strcpy(	monsters[5][6]->death, "");

    strcpy(	monsters[5][7]->name, "");
    strcpy(	monsters[5][7]->weapon, "");
		monsters[5][7]->strength = 1;
		monsters[5][7]->gold = 1;
		monsters[5][7]->exp = 1;
		monsters[5][7]->maxhp = 1;
    strcpy(	monsters[5][7]->death, "");

    strcpy(	monsters[5][8]->name, "");
    strcpy(	monsters[5][8]->weapon, "");
		monsters[5][8]->strength = 1;
		monsters[5][8]->gold = 1;
		monsters[5][8]->exp = 1;
		monsters[5][8]->maxhp = 1;
    strcpy(	monsters[5][8]->death, "");

    strcpy(	monsters[5][9]->name, "");
    strcpy(	monsters[5][9]->weapon, "");
		monsters[5][9]->strength = 1;
		monsters[5][9]->gold = 1;
		monsters[5][9]->exp = 1;
		monsters[5][9]->maxhp = 1;
    strcpy(	monsters[5][9]->death, "");

    strcpy(	monsters[5][10]->name, "");
    strcpy(	monsters[5][10]->weapon, "");
		monsters[5][10]->strength = 1;
		monsters[5][10]->gold = 1;
		monsters[5][10]->exp = 1;
		monsters[5][10]->maxhp = 1;
    strcpy(	monsters[5][10]->death, "");

    strcpy(	monsters[5][11]->name, "");
    strcpy(	monsters[5][11]->weapon, "");
		monsters[5][11]->strength = 1;
		monsters[5][11]->gold = 1;
		monsters[5][11]->exp = 1;
		monsters[5][11]->maxhp = 1;
    strcpy(	monsters[5][11]->death, "");

    strcpy(	monsters[6][0]->name, "");
    strcpy(	monsters[6][0]->weapon, "");
		monsters[6][0]->strength = 1;
		monsters[6][0]->gold = 1;
		monsters[6][0]->exp = 1;
		monsters[6][0]->maxhp = 1;
    strcpy(	monsters[6][0]->death, "");

    strcpy(	monsters[6][1]->name, "");
    strcpy(	monsters[6][1]->weapon, "");
		monsters[6][1]->strength = 1;
		monsters[6][1]->gold = 1;
		monsters[6][1]->exp = 1;
		monsters[6][1]->maxhp = 1;
    strcpy(	monsters[6][1]->death, "");

    strcpy(	monsters[6][2]->name, "");
    strcpy(	monsters[6][2]->weapon, "");
		monsters[6][2]->strength = 1;
		monsters[6][2]->gold = 1;
		monsters[6][2]->exp = 1;
		monsters[6][2]->maxhp = 1;
    strcpy(	monsters[6][2]->death, "");

    strcpy(	monsters[6][3]->name, "");
    strcpy(	monsters[6][3]->weapon, "");
		monsters[6][3]->strength = 1;
		monsters[6][3]->gold = 1;
		monsters[6][3]->exp = 1;
		monsters[6][3]->maxhp = 1;
    strcpy(	monsters[6][3]->death, "");

    strcpy(	monsters[6][4]->name, "");
    strcpy(	monsters[6][4]->weapon, "");
		monsters[6][4]->strength = 1;
		monsters[6][4]->gold = 1;
		monsters[6][4]->exp = 1;
		monsters[6][4]->maxhp = 1;
    strcpy(	monsters[6][4]->death, "");

    strcpy(	monsters[6][5]->name, "");
    strcpy(	monsters[6][5]->weapon, "");
		monsters[6][5]->strength = 1;
		monsters[6][5]->gold = 1;
		monsters[6][5]->exp = 1;
		monsters[6][5]->maxhp = 1;
    strcpy(	monsters[6][5]->death, "");

    strcpy(	monsters[6][6]->name, "");
    strcpy(	monsters[6][6]->weapon, "");
		monsters[6][6]->strength = 1;
		monsters[6][6]->gold = 1;
		monsters[6][6]->exp = 1;
		monsters[6][6]->maxhp = 1;
    strcpy(	monsters[6][6]->death, "");

    strcpy(	monsters[6][7]->name, "");
    strcpy(	monsters[6][7]->weapon, "");
		monsters[6][7]->strength = 1;
		monsters[6][7]->gold = 1;
		monsters[6][7]->exp = 1;
		monsters[6][7]->maxhp = 1;
    strcpy(	monsters[6][7]->death, "");

    strcpy(	monsters[6][8]->name, "");
    strcpy(	monsters[6][8]->weapon, "");
		monsters[6][8]->strength = 1;
		monsters[6][8]->gold = 1;
		monsters[6][8]->exp = 1;
		monsters[6][8]->maxhp = 1;
    strcpy(	monsters[6][8]->death, "");

    strcpy(	monsters[6][9]->name, "");
    strcpy(	monsters[6][9]->weapon, "");
		monsters[6][9]->strength = 1;
		monsters[6][9]->gold = 1;
		monsters[6][9]->exp = 1;
		monsters[6][9]->maxhp = 1;
    strcpy(	monsters[6][9]->death, "");

    strcpy(	monsters[6][10]->name, "");
    strcpy(	monsters[6][10]->weapon, "");
		monsters[6][10]->strength = 1;
		monsters[6][10]->gold = 1;
		monsters[6][10]->exp = 1;
		monsters[6][10]->maxhp = 1;
    strcpy(	monsters[6][10]->death, "");

    strcpy(	monsters[6][11]->name, "");
    strcpy(	monsters[6][11]->weapon, "");
		monsters[6][11]->strength = 1;
		monsters[6][11]->gold = 1;
		monsters[6][11]->exp = 1;
		monsters[6][11]->maxhp = 1;
    strcpy(	monsters[6][11]->death, "");

    strcpy(	monsters[7][0]->name, "");
    strcpy(	monsters[7][0]->weapon, "");
		monsters[7][0]->strength = 1;
		monsters[7][0]->gold = 1;
		monsters[7][0]->exp = 1;
		monsters[7][0]->maxhp = 1;
    strcpy(	monsters[7][0]->death, "");

    strcpy(	monsters[7][1]->name, "");
    strcpy(	monsters[7][1]->weapon, "");
		monsters[7][1]->strength = 1;
		monsters[7][1]->gold = 1;
		monsters[7][1]->exp = 1;
		monsters[7][1]->maxhp = 1;
    strcpy(	monsters[7][1]->death, "");

    strcpy(	monsters[7][2]->name, "");
    strcpy(	monsters[7][2]->weapon, "");
		monsters[7][2]->strength = 1;
		monsters[7][2]->gold = 1;
		monsters[7][2]->exp = 1;
		monsters[7][2]->maxhp = 1;
    strcpy(	monsters[7][2]->death, "");

    strcpy(	monsters[7][3]->name, "");
    strcpy(	monsters[7][3]->weapon, "");
		monsters[7][3]->strength = 1;
		monsters[7][3]->gold = 1;
		monsters[7][3]->exp = 1;
		monsters[7][3]->maxhp = 1;
    strcpy(	monsters[7][3]->death, "");

    strcpy(	monsters[7][4]->name, "");
    strcpy(	monsters[7][4]->weapon, "");
		monsters[7][4]->strength = 1;
		monsters[7][4]->gold = 1;
		monsters[7][4]->exp = 1;
		monsters[7][4]->maxhp = 1;
    strcpy(	monsters[7][4]->death, "");

    strcpy(	monsters[7][5]->name, "");
    strcpy(	monsters[7][5]->weapon, "");
		monsters[7][5]->strength = 1;
		monsters[7][5]->gold = 1;
		monsters[7][5]->exp = 1;
		monsters[7][5]->maxhp = 1;
    strcpy(	monsters[7][5]->death, "");

    strcpy(	monsters[7][6]->name, "");
    strcpy(	monsters[7][6]->weapon, "");
		monsters[7][6]->strength = 1;
		monsters[7][6]->gold = 1;
		monsters[7][6]->exp = 1;
		monsters[7][6]->maxhp = 1;
    strcpy(	monsters[7][6]->death, "");

    strcpy(	monsters[7][7]->name, "");
    strcpy(	monsters[7][7]->weapon, "");
		monsters[7][7]->strength = 1;
		monsters[7][7]->gold = 1;
		monsters[7][7]->exp = 1;
		monsters[7][7]->maxhp = 1;
    strcpy(	monsters[7][7]->death, "");

    strcpy(	monsters[7][8]->name, "");
    strcpy(	monsters[7][8]->weapon, "");
		monsters[7][8]->strength = 1;
		monsters[7][8]->gold = 1;
		monsters[7][8]->exp = 1;
		monsters[7][8]->maxhp = 1;
    strcpy(	monsters[7][8]->death, "");

    strcpy(	monsters[7][9]->name, "");
    strcpy(	monsters[7][9]->weapon, "");
		monsters[7][9]->strength = 1;
		monsters[7][9]->gold = 1;
		monsters[7][9]->exp = 1;
		monsters[7][9]->maxhp = 1;
    strcpy(	monsters[7][9]->death, "");

    strcpy(	monsters[7][10]->name, "");
    strcpy(	monsters[7][10]->weapon, "");
		monsters[7][10]->strength = 1;
		monsters[7][10]->gold = 1;
		monsters[7][10]->exp = 1;
		monsters[7][10]->maxhp = 1;
    strcpy(	monsters[7][10]->death, "");

    strcpy(	monsters[7][11]->name, "");
    strcpy(	monsters[7][11]->weapon, "");
		monsters[7][11]->strength = 1;
		monsters[7][11]->gold = 1;
		monsters[7][11]->exp = 1;
		monsters[7][11]->maxhp = 1;
    strcpy(	monsters[7][11]->death, "");

    strcpy(	monsters[8][0]->name, "");
    strcpy(	monsters[8][0]->weapon, "");
		monsters[8][0]->strength = 1;
		monsters[8][0]->gold = 1;
		monsters[8][0]->exp = 1;
		monsters[8][0]->maxhp = 1;
    strcpy(	monsters[8][0]->death, "");

    strcpy(	monsters[8][1]->name, "");
    strcpy(	monsters[8][1]->weapon, "");
		monsters[8][1]->strength = 1;
		monsters[8][1]->gold = 1;
		monsters[8][1]->exp = 1;
		monsters[8][1]->maxhp = 1;
    strcpy(	monsters[8][1]->death, "");

    strcpy(	monsters[8][2]->name, "");
    strcpy(	monsters[8][2]->weapon, "");
		monsters[8][2]->strength = 1;
		monsters[8][2]->gold = 1;
		monsters[8][2]->exp = 1;
		monsters[8][2]->maxhp = 1;
    strcpy(	monsters[8][2]->death, "");

    strcpy(	monsters[8][3]->name, "");
    strcpy(	monsters[8][3]->weapon, "");
		monsters[8][3]->strength = 1;
		monsters[8][3]->gold = 1;
		monsters[8][3]->exp = 1;
		monsters[8][3]->maxhp = 1;
    strcpy(	monsters[8][3]->death, "");

    strcpy(	monsters[8][4]->name, "");
    strcpy(	monsters[8][4]->weapon, "");
		monsters[8][4]->strength = 1;
		monsters[8][4]->gold = 1;
		monsters[8][4]->exp = 1;
		monsters[8][4]->maxhp = 1;
    strcpy(	monsters[8][4]->death, "");

    strcpy(	monsters[8][5]->name, "");
    strcpy(	monsters[8][5]->weapon, "");
		monsters[8][5]->strength = 1;
		monsters[8][5]->gold = 1;
		monsters[8][5]->exp = 1;
		monsters[8][5]->maxhp = 1;
    strcpy(	monsters[8][5]->death, "");

    strcpy(	monsters[8][6]->name, "");
    strcpy(	monsters[8][6]->weapon, "");
		monsters[8][6]->strength = 1;
		monsters[8][6]->gold = 1;
		monsters[8][6]->exp = 1;
		monsters[8][6]->maxhp = 1;
    strcpy(	monsters[8][6]->death, "");

    strcpy(	monsters[8][7]->name, "");
    strcpy(	monsters[8][7]->weapon, "");
		monsters[8][7]->strength = 1;
		monsters[8][7]->gold = 1;
		monsters[8][7]->exp = 1;
		monsters[8][7]->maxhp = 1;
    strcpy(	monsters[8][7]->death, "");

    strcpy(	monsters[8][8]->name, "");
    strcpy(	monsters[8][8]->weapon, "");
		monsters[8][8]->strength = 1;
		monsters[8][8]->gold = 1;
		monsters[8][8]->exp = 1;
		monsters[8][8]->maxhp = 1;
    strcpy(	monsters[8][8]->death, "");

    strcpy(	monsters[8][9]->name, "");
    strcpy(	monsters[8][9]->weapon, "");
		monsters[8][9]->strength = 1;
		monsters[8][9]->gold = 1;
		monsters[8][9]->exp = 1;
		monsters[8][9]->maxhp = 1;
    strcpy(	monsters[8][9]->death, "");

    strcpy(	monsters[8][10]->name, "");
    strcpy(	monsters[8][10]->weapon, "");
		monsters[8][10]->strength = 1;
		monsters[8][10]->gold = 1;
		monsters[8][10]->exp = 1;
		monsters[8][10]->maxhp = 1;
    strcpy(	monsters[8][10]->death, "");

    strcpy(	monsters[8][11]->name, "");
    strcpy(	monsters[8][11]->weapon, "");
		monsters[8][11]->strength = 1;
		monsters[8][11]->gold = 1;
		monsters[8][11]->exp = 1;
		monsters[8][11]->maxhp = 1;
    strcpy(	monsters[8][11]->death, "");

    strcpy(	monsters[9][0]->name, "");
    strcpy(	monsters[9][0]->weapon, "");
		monsters[9][0]->strength = 1;
		monsters[9][0]->gold = 1;
		monsters[9][0]->exp = 1;
		monsters[9][0]->maxhp = 1;
    strcpy(	monsters[9][0]->death, "");

    strcpy(	monsters[9][1]->name, "");
    strcpy(	monsters[9][1]->weapon, "");
		monsters[9][1]->strength = 1;
		monsters[9][1]->gold = 1;
		monsters[9][1]->exp = 1;
		monsters[9][1]->maxhp = 1;
    strcpy(	monsters[9][1]->death, "");

    strcpy(	monsters[9][2]->name, "");
    strcpy(	monsters[9][2]->weapon, "");
		monsters[9][2]->strength = 1;
		monsters[9][2]->gold = 1;
		monsters[9][2]->exp = 1;
		monsters[9][2]->maxhp = 1;
    strcpy(	monsters[9][2]->death, "");

    strcpy(	monsters[9][3]->name, "");
    strcpy(	monsters[9][3]->weapon, "");
		monsters[9][3]->strength = 1;
		monsters[9][3]->gold = 1;
		monsters[9][3]->exp = 1;
		monsters[9][3]->maxhp = 1;
    strcpy(	monsters[9][3]->death, "");

    strcpy(	monsters[9][4]->name, "");
    strcpy(	monsters[9][4]->weapon, "");
		monsters[9][4]->strength = 1;
		monsters[9][4]->gold = 1;
		monsters[9][4]->exp = 1;
		monsters[9][4]->maxhp = 1;
    strcpy(	monsters[9][4]->death, "");

    strcpy(	monsters[9][5]->name, "");
    strcpy(	monsters[9][5]->weapon, "");
		monsters[9][5]->strength = 1;
		monsters[9][5]->gold = 1;
		monsters[9][5]->exp = 1;
		monsters[9][5]->maxhp = 1;
    strcpy(	monsters[9][5]->death, "");

    strcpy(	monsters[9][6]->name, "");
    strcpy(	monsters[9][6]->weapon, "");
		monsters[9][6]->strength = 1;
		monsters[9][6]->gold = 1;
		monsters[9][6]->exp = 1;
		monsters[9][6]->maxhp = 1;
    strcpy(	monsters[9][6]->death, "");

    strcpy(	monsters[9][7]->name, "");
    strcpy(	monsters[9][7]->weapon, "");
		monsters[9][7]->strength = 1;
		monsters[9][7]->gold = 1;
		monsters[9][7]->exp = 1;
		monsters[9][7]->maxhp = 1;
    strcpy(	monsters[9][7]->death, "");

    strcpy(	monsters[9][8]->name, "");
    strcpy(	monsters[9][8]->weapon, "");
		monsters[9][8]->strength = 1;
		monsters[9][8]->gold = 1;
		monsters[9][8]->exp = 1;
		monsters[9][8]->maxhp = 1;
    strcpy(	monsters[9][8]->death, "");

    strcpy(	monsters[9][9]->name, "");
    strcpy(	monsters[9][9]->weapon, "");
		monsters[9][9]->strength = 1;
		monsters[9][9]->gold = 1;
		monsters[9][9]->exp = 1;
		monsters[9][9]->maxhp = 1;
    strcpy(	monsters[9][9]->death, "");

    strcpy(	monsters[9][10]->name, "");
    strcpy(	monsters[9][10]->weapon, "");
		monsters[9][10]->strength = 1;
		monsters[9][10]->gold = 1;
		monsters[9][10]->exp = 1;
		monsters[9][10]->maxhp = 1;
    strcpy(	monsters[9][10]->death, "");

    strcpy(	monsters[9][11]->name, "");
    strcpy(	monsters[9][11]->weapon, "");
		monsters[9][11]->strength = 1;
		monsters[9][11]->gold = 1;
		monsters[9][11]->exp = 1;
		monsters[9][11]->maxhp = 1;
    strcpy(	monsters[9][11]->death, "");

    strcpy(	monsters[10][0]->name, "");
    strcpy(	monsters[10][0]->weapon, "");
		monsters[10][0]->strength = 1;
		monsters[10][0]->gold = 1;
		monsters[10][0]->exp = 1;
		monsters[10][0]->maxhp = 1;
    strcpy(	monsters[10][0]->death, "");

    strcpy(	monsters[10][1]->name, "");
    strcpy(	monsters[10][1]->weapon, "");
		monsters[10][1]->strength = 1;
		monsters[10][1]->gold = 1;
		monsters[10][1]->exp = 1;
		monsters[10][1]->maxhp = 1;
    strcpy(	monsters[10][1]->death, "");

    strcpy(	monsters[10][2]->name, "");
    strcpy(	monsters[10][2]->weapon, "");
		monsters[10][2]->strength = 1;
		monsters[10][2]->gold = 1;
		monsters[10][2]->exp = 1;
		monsters[10][2]->maxhp = 1;
    strcpy(	monsters[10][2]->death, "");

    strcpy(	monsters[10][3]->name, "");
    strcpy(	monsters[10][3]->weapon, "");
		monsters[10][3]->strength = 1;
		monsters[10][3]->gold = 1;
		monsters[10][3]->exp = 1;
		monsters[10][3]->maxhp = 1;
    strcpy(	monsters[10][3]->death, "");

    strcpy(	monsters[10][4]->name, "");
    strcpy(	monsters[10][4]->weapon, "");
		monsters[10][4]->strength = 1;
		monsters[10][4]->gold = 1;
		monsters[10][4]->exp = 1;
		monsters[10][4]->maxhp = 1;
    strcpy(	monsters[10][4]->death, "");

    strcpy(	monsters[10][5]->name, "");
    strcpy(	monsters[10][5]->weapon, "");
		monsters[10][5]->strength = 1;
		monsters[10][5]->gold = 1;
		monsters[10][5]->exp = 1;
		monsters[10][5]->maxhp = 1;
    strcpy(	monsters[10][5]->death, "");

    strcpy(	monsters[10][6]->name, "");
    strcpy(	monsters[10][6]->weapon, "");
		monsters[10][6]->strength = 1;
		monsters[10][6]->gold = 1;
		monsters[10][6]->exp = 1;
		monsters[10][6]->maxhp = 1;
    strcpy(	monsters[10][6]->death, "");

    strcpy(	monsters[10][7]->name, "");
    strcpy(	monsters[10][7]->weapon, "");
		monsters[10][7]->strength = 1;
		monsters[10][7]->gold = 1;
		monsters[10][7]->exp = 1;
		monsters[10][7]->maxhp = 1;
    strcpy(	monsters[10][7]->death, "");

    strcpy(	monsters[10][8]->name, "");
    strcpy(	monsters[10][8]->weapon, "");
		monsters[10][8]->strength = 1;
		monsters[10][8]->gold = 1;
		monsters[10][8]->exp = 1;
		monsters[10][8]->maxhp = 1;
    strcpy(	monsters[10][8]->death, "");

    strcpy(	monsters[10][9]->name, "");
    strcpy(	monsters[10][9]->weapon, "");
		monsters[10][9]->strength = 1;
		monsters[10][9]->gold = 1;
		monsters[10][9]->exp = 1;
		monsters[10][9]->maxhp = 1;
    strcpy(	monsters[10][9]->death, "");

    strcpy(	monsters[10][10]->name, "");
    strcpy(	monsters[10][10]->weapon, "");
		monsters[10][10]->strength = 1;
		monsters[10][10]->gold = 1;
		monsters[10][10]->exp = 1;
		monsters[10][10]->maxhp = 1;
    strcpy(	monsters[10][10]->death, "");

    strcpy(	monsters[10][11]->name, "");
    strcpy(	monsters[10][11]->weapon, "");
		monsters[10][11]->strength = 1;
		monsters[10][11]->gold = 1;
		monsters[10][11]->exp = 1;
		monsters[10][11]->maxhp = 1;
    strcpy(	monsters[10][11]->death, "");

    strcpy(	monsters[11][0]->name, "");
    strcpy(	monsters[11][0]->weapon, "");
		monsters[11][0]->strength = 1;
		monsters[11][0]->gold = 1;
		monsters[11][0]->exp = 1;
		monsters[11][0]->maxhp = 1;
    strcpy(	monsters[11][0]->death, "");

    strcpy(	monsters[11][1]->name, "");
    strcpy(	monsters[11][1]->weapon, "");
		monsters[11][1]->strength = 1;
		monsters[11][1]->gold = 1;
		monsters[11][1]->exp = 1;
		monsters[11][1]->maxhp = 1;
    strcpy(	monsters[11][1]->death, "");

    strcpy(	monsters[11][2]->name, "");
    strcpy(	monsters[11][2]->weapon, "");
		monsters[11][2]->strength = 1;
		monsters[11][2]->gold = 1;
		monsters[11][2]->exp = 1;
		monsters[11][2]->maxhp = 1;
    strcpy(	monsters[11][2]->death, "");

    strcpy(	monsters[11][3]->name, "");
    strcpy(	monsters[11][3]->weapon, "");
		monsters[11][3]->strength = 1;
		monsters[11][3]->gold = 1;
		monsters[11][3]->exp = 1;
		monsters[11][3]->maxhp = 1;
    strcpy(	monsters[11][3]->death, "");

    strcpy(	monsters[11][4]->name, "");
    strcpy(	monsters[11][4]->weapon, "");
		monsters[11][4]->strength = 1;
		monsters[11][4]->gold = 1;
		monsters[11][4]->exp = 1;
		monsters[11][4]->maxhp = 1;
    strcpy(	monsters[11][4]->death, "");

    strcpy(	monsters[11][5]->name, "");
    strcpy(	monsters[11][5]->weapon, "");
		monsters[11][5]->strength = 1;
		monsters[11][5]->gold = 1;
		monsters[11][5]->exp = 1;
		monsters[11][5]->maxhp = 1;
    strcpy(	monsters[11][5]->death, "");

    strcpy(	monsters[11][6]->name, "");
    strcpy(	monsters[11][6]->weapon, "");
		monsters[11][6]->strength = 1;
		monsters[11][6]->gold = 1;
		monsters[11][6]->exp = 1;
		monsters[11][6]->maxhp = 1;
    strcpy(	monsters[11][6]->death, "");

    strcpy(	monsters[11][7]->name, "");
    strcpy(	monsters[11][7]->weapon, "");
		monsters[11][7]->strength = 1;
		monsters[11][7]->gold = 1;
		monsters[11][7]->exp = 1;
		monsters[11][7]->maxhp = 1;
    strcpy(	monsters[11][7]->death, "");

    strcpy(	monsters[11][8]->name, "");
    strcpy(	monsters[11][8]->weapon, "");
		monsters[11][8]->strength = 1;
		monsters[11][8]->gold = 1;
		monsters[11][8]->exp = 1;
		monsters[11][8]->maxhp = 1;
    strcpy(	monsters[11][8]->death, "");

    strcpy(	monsters[11][9]->name, "");
    strcpy(	monsters[11][9]->weapon, "");
		monsters[11][9]->strength = 1;
		monsters[11][9]->gold = 1;
		monsters[11][9]->exp = 1;
		monsters[11][9]->maxhp = 1;
    strcpy(	monsters[11][9]->death, "");

    strcpy(	monsters[11][10]->name, "");
    strcpy(	monsters[11][10]->weapon, "");
		monsters[11][10]->strength = 1;
		monsters[11][10]->gold = 1;
		monsters[11][10]->exp = 1;
		monsters[11][10]->maxhp = 1;
    strcpy(	monsters[11][10]->death, "");

    strcpy(	monsters[11][11]->name, "");
    strcpy(	monsters[11][11]->weapon, "");
		monsters[11][11]->strength = 1;
		monsters[11][11]->gold = 1;
		monsters[11][11]->exp = 1;
		monsters[11][11]->maxhp = 1;
    strcpy(	monsters[11][11]->death, "");

*/
}

void delete_monsters()
{
    for (int x = 0; x < LEVELS; x++)
	for (int y = 0; y < MONSTERS; y++)
	    if (monsters[x][y])
		delete monsters[x][y];
}

void delete_masters()
{
    for (int x = 0; x < LEVELS; x++)
	if (masters[x])
	    delete masters[x];
}

void display_monster(char *u)
{
    if (is_playing(u))
    {
	aClient *user = find(u);
	Player *ni = user->stats;
	
	notice(s_GameServ, u, "Your Hitpoints: %d", ni->hp);
	notice(s_GameServ, u, "%s's Hitpoints: %d", ni->fight->name, ni->fight->hp);
	notice(s_GameServ, u, "Here are your commands:");
	notice(s_GameServ, u, "/msg %S attack");
	notice(s_GameServ, u, "/msg %S run");
	notice(s_GameServ, u, "What will you do?");
    }
}

void display_players(char *u)
{
    if (is_playing(u))
    {
	aClient *ni = find(u);

    	aClient *battle = ni->stats->battle;

	notice(s_GameServ, u, "Your Hitpoints: %d", ni->stats->hp);
	notice(s_GameServ, u, "%s's Hitpoints: %d", battle->getNick(), 
							battle->stats->hp);

	notice(s_GameServ, u, "Here are your commands:");
	notice(s_GameServ, u, "/msg %s attack", s_GameServ);
	notice(s_GameServ, u, "/msg %s run", s_GameServ);
	notice(s_GameServ, u, "What will you do?");
    }
}


bool is_playing(char *u)
{
    aClient *user;
    if (!(user = find(u)))
    {
	return false;
    }
    else
    {
	return user->stats != NULL;
    }
}

bool is_playing(aClient *user)
{
    return user->stats != NULL;
}

bool is_alive(char *u)
{
    aClient *user;
    if (!(user = find(u)))
	return false;
    else if (user->stats == NULL)
	return false;
    else
	return user->stats->alive;
}

bool is_alive(aClient *user)
{
    if (user->stats == NULL)
	return false;
    else
	return user->stats->alive;
}
	
bool is_fighting(char *u)
{
    aClient *user;

    if (!(user = find(u)))
    {
	return false;
    }
    else if (user->stats)
    {
	return user->stats->fight != NULL || user->stats->battle != NULL 
		|| user->stats->master != NULL;
    }
    else
    	return false;
}
bool is_fighting(aClient *user)
{
    if (!is_playing(user))
	return false;
    else
	return (user->stats->fight != NULL || user->stats->battle != NULL || user->stats->master != NULL);
}

bool player_fight(char *u)
{
    aClient *user;

    if (!(user = find(u)))
	return false;
    else if (user->stats)
	return user->stats->battle != NULL;
    else
	return false;
}
bool player_fight(aClient *user)
{
    if (!is_fighting(user))
	return false;
    else
	return user->stats->battle != NULL;
}

bool master_fight(char *u)
{
    aClient *user;

    if (!(user = find(u)))
	return false;
    else if (user->stats)
	return user->stats->master != NULL;
    else
	return false;
}
bool master_fight(aClient *user)
{
    if (!is_playing(user))
	return false;
    else
	return user->stats->master != NULL;
}

void do_fight(char *u)
{
    aClient *ni, *battle;

    char *nick = strtok(NULL, " ");

    if (!nick)
    {
	notice(s_GameServ, u, "SYNTAX: /msg %S FIGHT PLAYER");
    }
    else if (!(ni = find(u)))
    {
	notice(s_GameServ, u, "Fatal error. Contact a(n) %S admin. buf: %s", strtok(NULL, ""));
	return;
    }
    else if (!(battle = find(nick)))
    {
	notice(s_GameServ, u, "You can't attack %s while they aren't playing!", nick);
    }
    else if (!is_playing(ni))
    {
	notice(s_GameServ, u, "You are not playing!");
    }
/*
 * Offline fighting not implemented yet.
 *   else if (!(fight = finduser(nick)))
 *   {
 *       ni->stats->battle = battle;
 *       battle->battle = ni;
 *       ni->yourturn = 1;
 *       battle->yourturn = 0;
 *       notice(s_GameServ, u, "You decide to fight %s while they're not online!",
 *                battle->getNick());
 *       display_players(u);
 *   }
 */
    else if (!is_alive(ni))
    {
	notice(s_GameServ, u, "You are dead. Wait until tomorrow to fight others!");
	return;
    }
    else if (is_playing(ni) && is_playing(battle) && stricmp(ni->stats->name, battle->stats->name) != 0)
    {
	// Set your battle pointer to the other player
        ni->stats->battle = battle;

	// Set the other player's battle pointer to you
        battle->stats->battle = ni;

	// The initiator gets the first move (perhaps this should be 50/50)
        ni->stats->yourturn = 1;
        battle->stats->yourturn = 0;

	// Initiate Battle sequence!
        notice(s_GameServ, u, "You challenge %s to an online duel!", battle->getNick());
        notice(s_GameServ, battle->getNick(), "%s has challenged you to an online duel!", u);
        notice(s_GameServ, battle->getNick(), "%s gets to go first because he initiated!", u);
        notice(s_GameServ, battle->getNick(), "Please wait while %s decides what to do.", u);
        display_players(u);
    }
}
void do_run(char *u)
{
    aClient *user;
    Player *p, *p2 = NULL;

    if (!(user = find(u)))
    {
	notice(s_GameServ, u, "Couldn't find you. Error. Contact a %S admin");
	return;
    }

    p = user->stats;

    if (p->battle)
	p2 = p->battle->stats;

    if (!is_fighting(user))
	notice(s_GameServ, u, "You run in place... try fighting next time.");
    else if (!player_fight(user) && !master_fight(user))
    {
	notice(s_GameServ, u, "You run away from %s like a little baby!", p->fight->name);
	delete p->fight;
	p->fight = NULL;
    }
    else if (player_fight(user) && p->yourturn)
    {
	notice(s_GameServ, u, "You run away from %s like a little baby!", p2->name);
	notice(s_GameServ, p->battle->getNick(), "%s ran away from you like a little baby!", p->name);
	p2->battle = NULL;
    }
    else if (player_fight(user) && !p->yourturn)
    {
	notice(s_GameServ, u, "It is not your turn. Please wait until %s decides what to do.", p2->name);
    }
    else if (master_fight(user))
    {
	notice(s_GameServ, u, "You cannot run from %s! FIGHT!", p->master->name);
    }
    p->battle = NULL;
}
void do_attack(char *u)
{
    int hit, mhit;
    aClient *ni, *battle; // The player and perhaps the player they're fighting
    Monster *fight; // The monster they may be fighting

    if (!(ni = find(u)))
    {
	notice(s_GameServ, u, "You're not playing!");
	return;
    }
    else if (!ni->stats->fight && !ni->stats->battle && !ni->stats->master)
    {
	notice(s_GameServ, u, "You're not in battle!");
	return;
    }
    else
    {
	if (!ni->stats->master) // This is not a master fight
		fight = ni->stats->fight;	// Monster	Could be NULL
	else			// This IS a master fight
		fight = ni->stats->master;	// Master	Could be NULL

	battle = ni->stats->battle;		// Player	Could be NULL

	// One has to be !NULL based on the previous else if
	// We wouldn't be here if they were all NULL
    }

    if (!player_fight(ni))
    {
	// Player's Hit
        hit = ((ni->stats->strength + webonus[ni->stats->weapon]) / 2) +
              (rand() % ((ni->stats->strength + webonus[ni->stats->weapon]) / 2));

	// Opponent's Hit
        mhit = (fight->strength / 2) +
               (rand() % (fight->strength / 2) - (ni->stats->defense +
                arbonus[ni->stats->armor]));
    }
    else
    {
	// Opponent's Hit
        mhit = (((battle->stats->strength + webonus[battle->stats->weapon]) / 2) +
               (rand() % ((battle->stats->strength + webonus[battle->stats->weapon])) / 2) -
               (ni->stats->defense + arbonus[ni->stats->armor]));

	// Player's Hit
        hit = (((ni->stats->strength + webonus[ni->stats->weapon]) / 2) +
               (rand() % ((ni->stats->strength + webonus[ni->stats->weapon])) / 2) -
               (battle->stats->defense + arbonus[battle->stats->armor]));
    }

  if (!player_fight(ni))
  {
    if (hit > 0)
        notice(s_GameServ, u, "You attack %s for %d points!", fight->name, hit);
    else
        notice(s_GameServ, u, "You miss %s completely!", fight->name);

    if (hit >= fight->hp)
    {
        if (master_fight(ni))
            notice(s_GameServ, u, "You have bested %s!", fight->name);
        else
            notice(s_GameServ, u, "You have killed %s!", fight->name);

        notice(s_GameServ, u, "%s", fight->death);
        notice(s_GameServ, u, "You recieve %d experience and %d gold!",
                 fight->exp, fight->gold);

	// If your new experience (or gold) will be greater than 2 billion,
	// then set your exp to 2bil. (2 billion max)... otherwise add them.
	// This could be a problem with overflowing out of the sign bit.
	// Unsigned long int maybe? Leave it for now.
        ni->stats->exp = ( (ni->stats->exp + fight->exp) > 2000000000 ? 2000000000 : 
				ni->stats->exp + fight->exp);
        ni->stats->gold = (ni->stats->gold + fight->gold > 2000000000 ? 2000000000 : 
				ni->stats->gold + fight->gold);


        if (master_fight(ni))
        {
            notice(s_GameServ, u, "You are now level %d!", ni->stats->level + 1);
            notice(s_GameServ, u, "You gain %d Strength, and %d Defense points!",
                     strbonus[ni->stats->level - 1], defbonus[ni->stats->level - 1]);

	    // Increase your level
            ni->stats->level++;

	    // Increase your maximum hit points
            ni->stats->maxhp += hpbonus[ni->stats->level - 1];

	    // Heal the player by setting hp to their max
            ni->stats->hp = ni->stats->maxhp;

	    // Add to your strength
            ni->stats->strength += strbonus[ni->stats->level - 1];

	    // Add to your defensive power
            ni->stats->defense += defbonus[ni->stats->level - 1];

	    // Clear the pointer for your master
            ni->stats->master = NULL;
        }

	// They're dead so remove the pointer
	delete ni->stats->fight;
        ni->stats->fight = NULL; 
	ni->stats->master = NULL;

        return;
    }
    else
    {
        if (hit > 0)
            fight->hp -= hit;
        if (mhit > 0)
        {
            notice(s_GameServ, u, "%s attacks with their %s for %d damage!",
                     fight->name, fight->weapon, mhit);
        }
        else if (mhit <= 0)
            notice(s_GameServ, u, "%s completely misses you!", fight->name);

        if (mhit >= ni->stats->hp)
        {
            if (!master_fight(ni))
            {
                notice(s_GameServ, u, "You have been killed by %s!", fight->name);
                notice(s_GameServ, u, "You lose all gold on hand and lose 10 percent "\
                        "of your experience!");
                ni->stats->gold = 0;
                ni->stats->exp -= (long int)(ni->stats->exp * .10);
                ni->stats->fight = NULL;
		ni->stats->alive = false;
                return;
            }
            else
            {
                notice(s_GameServ, u, "%s has bested you! You will have to wait "\
                        "until tomorrow to try again", ni->stats->master->name);
                ni->stats->fight = NULL;
                ni->stats->master = NULL;
		return;
            }
        }
        else
        {
            if (mhit > 0)
                ni->stats->hp -= mhit;
            display_monster(u);
            return;
        }
    }
  }
  else if (player_fight(ni))
  {
/* Offline fighting not available yet
   if (!(online = finduser(ni->stats->battle->nick)) || !nick_identified(online))
   {
    if (hit > 0)
        notice(s_GameServ, u, "You attack %s for %d points!", battle->nick, hit);
    else
        notice(s_GameServ, u, "You miss %s completely!", battle->nick);
    if (hit >= battle->stats->hp)
    {
        notice(s_GameServ, u, "You have killed %s!", battle->nick);
*        notice(s_GameServ, u, "You recieve %d experience and %ld gold!",
                (long int)(battle->stats->exp * .10), battle->stats->gold);
        if (2000000000 - ni->stats->exp > (long int)(battle->stats->exp * .10))
        {
            ni->stats->exp += (long int)(battle->stats->exp * .10);
            battle->stats->exp -= (long int)(battle->stats->exp * .10);
        }
*        else
        {
            battle->stats->exp -= (long int)(battle->stats->exp * .10);
            ni->stats->exp = 2000000000;
        }

        if (2000000000 - ni->stats->gold > battle->stats->gold)
        {
*            ni->stats->gold += battle->stats->gold;
            battle->stats->gold = 0;
        }
        else
        {
            battle->stats->gold = 2000000000 - ni->stats->gold;
            ni->stats->gold = 2000000000;
        }
*        ni->stats->battle->stats->alive = 0;
        ni->stats->battle->battle = NULL;
        ni->stats->battle = NULL;
        return;
    }
    else
    {
        if (hit > 0)
*            battle->stats->hp -= hit;
        if (mhit > 0)
        {
            notice(s_GameServ, u, "%s hits you with their %s for %d damage!",
                     battle->nick, weapons[battle->stats->weapon], mhit);
        }
        else if (mhit <= 0)
            notice(s_GameServ, u, "%s completely misses you!", battle->nick);
*
        if (mhit >= ni->stats->hp)
        {
            notice(s_GameServ, u, "You have been killed by %s!", battle->nick);
            if (2000000000 - battle->stats->gold > ni->stats->gold)
            {
                notice(s_GameServ, u, "%s took all your gold!", battle->nick);
                battle->stats->gold += ni->stats->gold;
*                ni->stats->gold = 0;
            }
            else
            {
                notice(s_GameServ, u, "You're lucky, %s couldn't carry all your gold.",
                        battle->nick);
                ni->stats->gold -= (2000000000 - battle->stats->gold);
                notice(s_GameServ, u, "You were left dead with %d gold.",
*                         (long int)ni->stats->gold);
                battle->stats->gold = 2000000000;
            }
            ni->stats->battle->battle = NULL;
            ni->stats->battle = NULL;
            ni->stats->alive = 0;
            return;
        }
*        else
        {
            if (mhit > 0)
                ni->stats->hp -= mhit;
            display_players(u);
            return;
        }
    }
   }
* end offline fighting */

   if (is_playing(battle->getNick()))
   {
    if (ni->stats->yourturn == 0)
    {
        notice(s_GameServ, u, "Please wait until %s decides what to do!", 
		battle->getNick());
        return;
    }
    if (hit > 0)
    {
        notice(s_GameServ, u, "You attack %s for %d points!", battle->getNick(), hit);

        notice(s_GameServ, battle->getNick(), "%s has hit you with their %s for "\
					      "%d damage!", u, weapons[ni->stats->weapon], 
					      hit);
        ni->stats->yourturn = 0;
        battle->stats->yourturn = 1;
        display_players(battle->getNick());
    }
    else
    {
        notice(s_GameServ, u, "You miss %s completely!", battle->getNick());
        notice(s_GameServ, battle->getNick(), "%s misses you completely!", u);
        ni->stats->yourturn = 0;
        battle->stats->yourturn = 1;
        display_players(battle->getNick());
    }
    if (hit >= battle->stats->hp)
    {
        notice(s_GameServ, u, "You have killed %s!", battle->getNick());
        notice(s_GameServ, u, "You recieve %d experience and %ld gold!",
                (long int)(battle->stats->exp * .10), battle->stats->gold);
        notice(s_GameServ, battle->getNick(), "You have been killed by %s!", u);
        battle->stats->hp = 0;
        battle->stats->alive = false;

        if (2000000000 - ni->stats->exp > (long int)(battle->stats->exp * .10))
        {
            ni->stats->exp += (long int)(battle->stats->exp * .10);
            battle->stats->exp -= (long int)(battle->stats->exp * .10);
        }
        else
        {
            battle->stats->exp -= (long int)(battle->stats->exp * .10);
            ni->stats->exp = 2000000000;
        }

        if (2000000000 - ni->stats->gold > battle->stats->gold)
        {
           notice(s_GameServ, battle->getNick(), "You lose ten percent of experience and "\
						 "all gold on hand!");
            ni->stats->gold += battle->stats->gold;
            battle->stats->gold = 0;
        }
        else
        {
            battle->stats->gold = 2000000000 - ni->stats->gold;
            notice(s_GameServ, battle->getNick(), "You lose ten percent of your experience!");

            notice(s_GameServ, battle->getNick(), "However, %s could not carry all of your "\
			"gold.", u);

            notice(s_GameServ, battle->getNick(), "Luckily, you still have %ld gold "\
			"left. All is not lost!", battle->stats->gold);

            ni->stats->gold = 2000000000;
        }
	battle->stats->battle = NULL;
        ni->stats->battle = NULL;
        return;
    }
    else
    {
        if (hit > 0)
            battle->stats->hp -= hit;
        //display_players(battle->getNick());
        ni->stats->yourturn = 0;
        battle->stats->yourturn = 1;
        notice(s_GameServ, u, "Please wait while %s decides what to do!", 
		battle->getNick());

        return;
    }
   }
  }
}
void do_heal(char *u)
{
    aClient *ni;
    char *amount = strtok(NULL, " ");
    int price, num;

    if (!amount)
    {
	notice(s_GameServ, u, "SYNTAX: /msg %S HEAL {ALL | #}");
    }
    else if (!(ni = find(u)))
    {
	notice(s_GameServ, u, "Fatal error. Contact a(n) %S admin. buf: %s", strtok(NULL, ""));
	return;
    }
    else if (!is_playing(ni))
    {
	notice(s_GameServ, u, "You aren't playing!");
	return;
    }
    else if (!is_alive(ni))
    {
	notice(s_GameServ, u, "You are dead. Wait until tomorrow for healing.");
	return;
    }
    else if (is_fighting(ni))
    {
	notice(s_GameServ, u, "You can't heal in battle!");
    }
    else if (ni->stats->hp >= ni->stats->maxhp)
    {
        notice(s_GameServ, u, "You don't need healing!");
    }
    else if (stricmp(amount, "ALL") == 0)
    {
        price = ni->stats->level * 3;
        if (ni->stats->gold < (ni->stats->maxhp - ni->stats->hp) * price)
        {
            notice(s_GameServ, u, "Healing %d points for %d gold per point.",
                     (long int)ni->stats->gold/price, price);
            ni->stats->hp += ni->stats->gold / price;
            ni->stats->gold %= price;
        }
        else
        {
            notice(s_GameServ, u, "Healing all possible points at %d gold "\
			"per point.", price);
            notice(s_GameServ, u, "%d points healed for %ld gold. HP at MAX!",
                     (ni->stats->maxhp - ni->stats->hp), 
		     (price * (ni->stats->maxhp - ni->stats->hp)) );
            ni->stats->gold -= price * (ni->stats->maxhp - ni->stats->hp);
            ni->stats->hp = ni->stats->maxhp;
        }
    }
    else if (isstringnum(amount))
    {
        num = stringtoint(amount);
        price = ni->stats->level * 3;
        if (ni->stats->gold < price * num)
        {
            notice(s_GameServ, u, "You only have enough gold to heal %d points!",
                     (long int)ni->stats->gold/price);
        }
        else if (num <= ni->stats->maxhp - ni->stats->hp)
        {
            notice(s_GameServ, u, "Healing %d points at %d gold per point.",
                     num, price);
            ni->stats->hp += num;
            ni->stats->gold -= num * price;
        }
        else if (num > ni->stats->maxhp - ni->stats->hp)
        {
            notice(s_GameServ, u, "Healing all possible points at %d gold "\
			"per point.", price);
            notice(s_GameServ, u, "%d points healed. HP at MAX!",
                     (ni->stats->maxhp - ni->stats->hp));
            ni->stats->gold -= price * (ni->stats->maxhp - ni->stats->hp);
            ni->stats->hp = ni->stats->maxhp;
        }
    }
    else if (amount[0] == '-')
        notice(s_GameServ, u, "You trying to cheat?");
    else
	notice(s_GameServ, u, "SYNTAX: /msg %S HEAL {ALL | #}");
}

int isstringnum(char *num)
{
    unsigned int x;
    for (x = 0; x < strlen(num); x++)
    {
        if ((int)num[x] < 48 || (int)num[x] > 57)
            return 0;
    }
return 1;
}

long int stringtoint(char *number)
{
    long int x, len = strlen(number), sum = 0;
    if (len == 1)
        return chartoint(number[0]);
    sum += chartoint(number[len - 1]);
    for (x = len - 2; x >= 0; x--)
        sum += chartoint(number[x]) * pow(10, abs(x - len + 1));
    return sum;
}

long int pow(int x, int y)
{
    long int value = 0;
    int count = 0;
    value += x;

    if (x != 0 && y != 0)
    {
        for (count = 1; count <= y - 1; count++)
            value *= x;
    }
    else
        return 1;
return value;
}

long int chartoint(char ch)
{
	if (int(ch) >= 48 && int(ch) <= 57)
		return int(ch) - 48;
	else
		return 0;
}

int save_gs_dbase()
{
    ListNode<aClient> *ptr = players.First();
    Player *it;
    ofstream outfile;

    outfile.open(playerdata);

    if (!outfile)
    {
	cout << "Error opening " << playerdata << endl;
	return 0;
    }

    while(ptr)
    {
	it = ptr->getData()->stats;
	outfile << it->name << ' ' << it->level << ' ' << it->exp << ' ' << it->gold << ' ' << it->bank << ' '
		<< it->hp << ' ' << it->maxhp << ' ' << it->strength << ' ' << it->defense << ' '
		<< it->armor << ' ' << it->weapon << ' ' << (it->alive ? "alive" : "dead") << ' '
		<< it->forest_fights << ' ' << it->player_fights <<  ' ' 
		<< it->getFlags() << ' ' << it->password << endl;
	ptr = ptr->Next();
    }
outfile.close();
return 1;
}

int load_gs_dbase()
{
    ifstream infile;
    aClient *temp;
    Player *p;
    char *alive, *tempname, *buf, *password;
    buf = new char[1023];

    infile.open(playerdata);

    if (infile.fail())
    {
	cout << "Error opening " << playerdata << endl;
	return 0;
    }

    while (infile.getline(buf, 1024, '\n'))
    {
	temp = new aClient;
	tempname = strtok(buf, " ");
	temp->stats = new Player(tempname);
	p = temp->stats;

	p->level = stringtoint(strtok(NULL, " "));
	p->exp = stringtoint(strtok(NULL, " "));
	p->gold = stringtoint(strtok(NULL, " "));
	p->bank = stringtoint(strtok(NULL, " "));
	p->hp = stringtoint(strtok(NULL, " "));
	p->maxhp = stringtoint(strtok(NULL, " "));
	p->strength = stringtoint(strtok(NULL, " "));
	p->defense = stringtoint(strtok(NULL, " "));
	p->armor = stringtoint(strtok(NULL, " "));
	p->weapon = stringtoint(strtok(NULL, " "));
	alive = strtok(NULL, " ");
	p->alive = (stricmp(alive, "ALIVE") == 0 ? true : false);
	p->forest_fights = stringtoint(strtok(NULL, " "));
	p->player_fights = stringtoint(strtok(NULL, " "));
	p->setFlags(stringtoint(strtok(NULL, " ")));
	password = strtok(NULL, " ");
	strcpy(p->password, password);
	temp->setNick("NULL");

	printf("%s %d %ld %ld %ld %d %d %d %d %d %d %s %d %d %s\n", p->name, p->level, 
	p->exp, p->gold, p->bank, p->hp, p->maxhp, p->strength, p->defense, p->armor, p->weapon, 
	alive, p->forest_fights, p->player_fights, p->password);

	players.insertAtBack(temp);
	delete temp;
    }
delete [] buf;
infile.close();
return 1;
}

bool passcmp(char *encrypted, char *plaintext)
{
    char salt[3];
    char *plaintext2, *plainToencrypt;
    bool same = false;

    plaintext2 = new char[strlen(encrypted) + strlen(plaintext)]; // Extra
    strcpy(plaintext2, plaintext);
   
    salt[0] = encrypted[0];
    salt[1] = encrypted[1];
    salt[3] = '\0';

    plainToencrypt = crypt(plaintext2, salt);

    same = (strcmp((const char *)encrypted, plainToencrypt) == 0 ? true : false);

    delete []plaintext2;

    return same;
}

bool check_password(char *name, char *plaintext)
{
    aClient *client;

    if (!(client = findplayer(name)))
	return false;
    else
    {
	return passcmp(client->stats->password, plaintext);
    }
}

void do_store(char *u)
{
    char *cmd = strtok(NULL, " ");
    char *item = strtok(NULL, " ");
    char *num = strtok(NULL, " ");
    char *space;
    int wep;
    aClient *user;
    Player *p;

    if (!cmd || !item)
    {
	notice(s_GameServ, u, "SYNTAX: STORE LIST {ARMOR | WEAPONS}");
	notice(s_GameServ, u, "        STORE SELL {ARMOR | WEAPON}");
	notice(s_GameServ, u, "        STORE BUY {ARMOR | WEAPON} NUMBER");
    }
    else if (!(user = find(u)) || !is_playing(user))
	notice(s_GameServ, u, "You must be playing to use the store!");
    else if (!is_alive(user))
    {
	notice(s_GameServ, u, "You are dead. Wait until tomorrow to purchase weapons and armor!");
	return;
    }
    else if (stricmp(cmd, "LIST") == 0)
    {
	if (stricmp(item, "WEAPONS") == 0)
	{
            notice(s_GameServ, u, "Welcome to Kain's Armory");
            notice(s_GameServ, u, "Here are the weapons we have available for the killing, sire:");
	    for (int x = 1; x < WNA; x++)
            {
                space = spaces(strlen(weapons[x]), ".");
                notice(s_GameServ, u, "%s%d. %s%s%d",(x < 10 ? " " : ""), x, weapons[x], space, prices[x - 1]);
                free(space);
            }
	    notice(s_GameServ, u, "To purchase a weapon, type /msg %S STORE BUY NUM.");
            notice(s_GameServ, u, "Where num. is the weapon number from the menu above.");

	}
	else if (stricmp(item, "ARMOR") == 0)
	{
	    notice(s_GameServ, u, "Welcome to Kain's Armory");
	    notice(s_GameServ, u, "I hope you enjoy the fine armor we have available for your protection:");
	    for (int x = 1; x < WNA; x++)
            {
		space = spaces(strlen(armors[x]), ".");
		notice(s_GameServ, u, "%s%d. %s%s%d",(x < 10 ? " " : ""), x, armors[x], space, prices[x - 1]);
		free(space);
	    }
            notice(s_GameServ, u, "To purchase armor, type /msg %S store buy armor num.");
            notice(s_GameServ, u, "Where num. is the armor number from the menu above.");


	}
    } else if (stricmp(cmd, "BUY") == 0) {
	if (!num)
	{
	    notice(s_GameServ, u, "SYNTAX: STORE BUY {ARMOR | WEAPON} NUMBER");
	    return;
	}
	else if (!isstringnum(num))
	{
	    notice(s_GameServ, u, "You must specify a number between 1 and %d. Not %s!", WNA - 1, num);
	    return;
	}
	if (stricmp(item, "WEAPON") == 0)
        {
            wep = stringtoint(num);
            if (wep >= WNA || wep < 1)
            {
		notice(s_GameServ, u, "The number %d is out of range. The number you provide must be between 1 and %d.", wep, WNA - 1);
                return;
            }

	    p = user->stats;

            if (p->weapon != 0)
                notice(s_GameServ, u, "You have to sell your %s first!", weapons[p->weapon]);
            else if (p->gold < prices[wep - 1])
                notice(s_GameServ, u, "You don't have enough gold for %s!", weapons[wep]);
            else
            {
                notice(s_GameServ, u, "You have purchased %s! Thanks for the gold!", weapons[wep]);
                p->weapon = wep;
                p->gold -= prices[wep - 1];
            }
        }
	else if (stricmp(item, "ARMOR") == 0)
        {
            wep = stringtoint(num);
            if (wep >= WNA || wep < 1)
            {
		notice(s_GameServ, u, "The number %d is out of range. The number you provide must be between 1 and %d.", wep, WNA - 1);
                return;
            }

	    p = user->stats;

            if (p->armor != 0)
                notice(s_GameServ, u, "You have to sell your %s first!", armors[p->armor]);
            else if (p->gold < prices[wep - 1])
                notice(s_GameServ, u, "You don't have enough gold for %s!", armors[wep]);
            else
            {
                notice(s_GameServ, u, "You have purchased %s! Thanks for the gold!", armors[wep]);
                p->armor = wep;
                p->gold -= prices[wep - 1];
            }
        }
    }
    else if (stricmp(cmd, "SELL" ) == 0)
    {
	p = user->stats;

        if (stricmp(item, "WEAPON") == 0)
        {
            if (p->weapon == 0)
            {
                notice(s_GameServ, u, "You want me to chop off your hands?");
                return;
            }
            else if (p->gold == 2000000000)
            {
                notice(s_GameServ, u, "You have enough gold. I'll just take that off your hands, sire.");
                p->weapon = 0;
            }
            else if (2000000000 - p->gold < (prices[p->weapon - 1] / 2))
            {
                notice(s_GameServ, u, "Thank you for your business! You now have as much gold as you can carry.");
                notice(s_GameServ, u, "However, you have no weapon... can I interest you in the %s?", weapons[WNA - 1]);
                p->gold = 2000000000;
                p->weapon = 0;
            }
            else
	    {
                notice(s_GameServ, u, "Thank you for your business! You now have %d more gold but no weapon!", (prices[p->weapon - 1] / 2));
                p->gold += (prices[p->weapon - 1] / 2);
                p->weapon = 0;
            }
        }
        else if (stricmp(item, "ARMOR") == 0)
        {
	    p = user->stats;

            if (p->armor == 0)
            {
                notice(s_GameServ, u, "I don't think you can be any more naked...");
                return;
            }
            if (p->gold == 2000000000)
            {
                notice(s_GameServ, u, "You have enough gold. I'll just take that off your hands, sire.");
                p->armor = 0;
            }
            else if (2000000000 - p->gold < (prices[p->armor - 1] / 2))
            {
                notice(s_GameServ, u, "Thank you for your business! You now have as much gold as you can carry.");
                notice(s_GameServ, u, "However, you have no armor... can I interest you in %s?", armors[WNA - 1]);
		p->gold = 2000000000;
                p->armor = 0;
            }
            else
            {
                notice(s_GameServ, u, "Thank you for your business! You now have %d more gold but no armor!",
                         (prices[p->armor - 1] / 2));

                p->gold += (prices[p->armor - 1] / 2);
                p->armor = 0;
            }
	}
        else
	{
	    notice(s_GameServ, u, "SYNTAX: STORE LIST {ARMOR | WEAPONS}");
	    notice(s_GameServ, u, "        STORE SELL {ARMOR | WEAPON}");
	    notice(s_GameServ, u, "        STORE BUY {ARMOR | WEAPON} NUMBER");
	}
    }
}

void do_bank(char *u)
{
    char *cmd = strtok(NULL, " ");
    char *amount = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");

    aClient *user;
    Player *p;

    if (!cmd || !amount || (stricmp(cmd, "TRANSFER") == 0 && !nick))
    {
	notice(s_GameServ, u, "BANK {WITHDRAW | DEPOSIT} {ALL | AMOUNT}");
	notice (s_GameServ, u, "BANK BALANCE");
	return;
    }

    user = find(u);
    if (!is_playing(user))
    {
	notice(s_GameServ, u, "You must be playing to use the bank!");
	return;
    }
    else if (!is_alive(user))
    {
	notice(s_GameServ, u, "You are dead. We don't accept gold from dead folk! Wait 'til tomorrow!");
	return;
    }
    else if (!isstringnum(amount) && stricmp(amount, "ALL") != 0)
    {
        notice(s_GameServ, u, "I don't know how to convert alphabet letters into currency, sire!");
        return;
    }

    p = user->stats;

    if (stricmp(cmd, "BALANCE") == 0)
    {
	showBankBalance(u);
    }
    else if (stricmp(cmd, "DEPOSIT") == 0)
    {
        if (p->bank == 2000000000)
        {
            notice(s_GameServ, u, "Your bank account is full, sire!");
            return;
        }
        else if (stricmp(amount, "ALL") == 0)
        {
            if (2000000000 - p->bank < p->gold)
            {
                notice(s_GameServ, u, "You don't have enough room for all of your gold.");
                notice(s_GameServ, u, "Depositing %ld gold into your account", (2000000000 - p->bank));
                p->gold -= (2000000000 - p->bank);
                p->bank = 2000000000;
		showBankBalance(u);
            }
            else
            {
                notice(s_GameServ, u, "Depositing %ld gold into your account!", p->gold);
                p->bank += p->gold;
                p->gold = 0;
		showBankBalance(u);
            }
        }
        else if (stringtoint(amount) > p->gold)
        {
            notice(s_GameServ, u, "Sire, you only have %ld gold!", p->gold);
	    showBankBalance(u);
            return;
        }
        else
        {
            if (2000000000 - p->bank < stringtoint(amount))
            {
                notice(s_GameServ, u, "You don't have room in your account for that much.");
                notice(s_GameServ, u, "Capping off your account with %ld gold!", (2000000000 - p->bank));
                p->gold -= (2000000000 - p->bank);
                p->bank = 2000000000;
		showBankBalance(u);
            }
            else
            {
                notice(s_GameServ, u, "Depositing %d gold into your account!", stringtoint(amount));
                p->bank += stringtoint(amount);
                p->gold -= stringtoint(amount);
		showBankBalance(u);
            }
        }
    }
    else if (stricmp(cmd, "WITHDRAW") == 0)
    {
        if (p->gold == 2000000000)
        {
            notice(s_GameServ, u, "You cannot carry any more gold, sire!");
	    showBankBalance(u);
            return;
        }
        else if (stricmp(amount, "ALL") == 0)
        {
            if (2000000000 - p->gold < p->bank)
            {
                notice(s_GameServ, u, "You don't have enough room to carry all that gold.");
                notice(s_GameServ, u, "Withdrawing %ld gold from your account", (2000000000 - p->gold));
                p->bank -= (2000000000 - p->gold);
                p->gold = 2000000000;
		showBankBalance(u);
            }
            else
            {
                notice(s_GameServ, u, "Withdrawing %ld gold from your account!", p->bank);
                p->gold += p->bank;
                p->bank = 0;
		showBankBalance(u);
            }
        }
        else if (stringtoint(amount) > p->bank)
        {
            notice(s_GameServ, u, "Sire, you only have %ld gold in the bank!", p->bank);
	    showBankBalance(u);
            return;
        }
        else
        {
            if (2000000000 - p->gold < stringtoint(amount))
            {
                notice(s_GameServ, u, "You don't enough have room to carry that much gold!");
                notice(s_GameServ, u, "You fill your pockets with %ld gold!",
                        (2000000000 - p->gold));
                p->bank -= (2000000000 - p->gold);
                p->gold = 2000000000;
		showBankBalance(u);
            }
            else
            {
                notice(s_GameServ, u, "Withdrawing %d gold from your account!", stringtoint(amount));
                p->gold += stringtoint(amount);
                p->bank -= stringtoint(amount);
		showBankBalance(u);
            }
        }
    }

}

void do_master(char *u)
{
    aClient *user;
    user = find(u);

    if (!user)
    {
	notice(s_GameServ, u, "Fatal error. Contact a(n) %S admin. buf: %s", strtok(NULL, ""));
	return;
    }
    else if (is_fighting(user))
    {
	notice(s_GameServ, u, "You're in the middle of a fight! Pay attention!");
	return;
    }
    else if (!is_alive(user))
    {
	notice(s_GameServ, u, "You're dead. Wait until tomorrow to see your master!");
	return;
    }
    else if (!is_playing(user))
    {
	notice(s_GameServ, u, "You must be playing to see your master!");
	return;
    }
    
    char *cmd = strtok(NULL, " ");
    Player *p = user->stats;
    long int need = 0;

    if (seenMaster(p))
    {
	notice(s_GameServ, u, "You have already seen your master today. Wait until tomorrow to try again");
	return;
    }

    if (cmd != NULL)
    {
	switch(p->level)
	{
	    case 1:
		need = 100;
		break;
	    case 2:
		need = 400;
		break;
	    case 3:
		need = 1000;
		break;
	    case 4:
		need = 4000;
		break;
	    case 5:
		need = 10000;
		break;
	    case 6:
		need = 40000;
		break;
	    case 7:
		need = 100000;
		break;
	    case 8:
		need = 400000;
		break;
	    case 9:
		need = 1000000;
		break;
	    case 10:
		need = 4000000;
		break;
	    case 11:
		need = 10000000;
		break;
	    case 12:
		need = p->exp + 1;
		notice(s_GameServ, u, "You are at level 12. You are the master. What's left? The DRAGON!");
		break;
	    default:
		need = p->exp + 1; // Unknown level... don't let them fight a fake master!
		break;
	}   
    }
    else
    {
	notice(s_GameServ, u, "SYNTAX: MASTER {FIGHT | QUESTION}");
	return;
    }

    if (stricmp(cmd, "FIGHT") == 0)
    {
	if (p->exp >= need)
	{
	    setMaster(p);
	    see_master(u);
	}
	else
	    notice(s_GameServ, u, "You are not worthy of fighting %s! You need %ld more experience.", masters[p->level - 1]->name, (need - p->exp));
	return;
    }
    else if (stricmp(cmd, "QUESTION") == 0)
    {
	if (p->exp >= need)
	    notice(s_GameServ, u, "%s looks you up and down and decides you are more ready than you will ever be.", masters[p->level - 1]->name);
	else
	    notice(s_GameServ, u, "You pathetic fool! You are no match for %s, %s!", masters[p->level - 1]->name, p->name);

	return;
    }
    else
    {
	notice(s_GameServ, u, "SYNTAX: MASTER {FIGHT | QUESTION}");
    }
}

void see_master(char *u)
{
    aClient *user;

    if (!(user = find(u)))
    {
	notice(s_GameServ, u, "Fatal error. Contact a(n) %S admin. buf: %s", strtok(NULL, ""));
	return;	
    }

    if (!is_fighting(user) && is_playing(user))
    {
	Player *p = user->stats;
	p->master = new Monster(masters[p->level - 1]);
	p->fight = p->master;
	display_monster(u);  // Since master is the same structure, use this function
    }
}

void showBankBalance(const char *u)
{
    aClient *user;
    Player *p;

    if (!(user = find(u)))
        return;

    p = user->stats;

    if (!p)
	return;

    notice(s_GameServ, u, "Account Balance: %ld     Gold On hand: %ld", p->bank, p->gold);

}

void refreshall()
{
    ListNode <aClient> *it;
    Player *p;

    it = players.First();

    while (it)
    {
	p = it->getData()->stats;
	refresh(p);
	it = it->Next();
    }
}

void refresh(Player *p)
{
    if (!p)
	return;

    p->hp = p->maxhp;
    p->forest_fights = 100;
    p->player_fights = 3;
    p->alive = true;
    clearMaster(p);
}

void do_refresh(char *u)
{
    char *nick = strtok(NULL, " ");
    aClient *user;

    if (!(user = find(u)))
    {
	notice(s_GameServ, u, "Error: aClient not found. Contact a %S admin");
	cout << "Error: aClient not found: " << u << endl;
	return;
    }
    else if (!isAdmin(user))
    {
	notice(s_GameServ, u, "You must be a %S admin to use this command!");
	return;
    }
    if (!nick)
    {
	notice(s_GameServ, u, "SYNTAX: REFRESH {ALL | NICK}");
	return;
    }
    else if (stricmp(nick, "ALL") == 0)
    {
	notice(s_GameServ, u, "Refreshing everyone's stats!");
	refreshall();
    }
    else if ((user = find(nick)))
    {
	if (is_playing(user))
	{
	    notice(s_GameServ, u, "Refreshing %s.", user->getNick());
	    refresh(user->stats);
	}
	else
	{
	    notice(s_GameServ, u, "%s is not playing.", user->getNick());
	}
    }
    else
    {
	notice(s_GameServ, u, "Nick %s not found.", nick);
	return;
    }
}

void do_help(char *u)
{
    char *cmd = strtok(NULL, " ");

    display_help(u, cmd);
}

void display_help(char *u, char *file)
{
    ifstream infile;
    char *buf;

    if (!file)
    {
	infile.open("helpfiles/help");
	if (infile.fail())
	{
	    cout << "Error opening helpfiles/help" << endl;
	    notice(s_GameServ, u, "Error opening helpfiles/help");
	    return;
	}
	buf = new char[1024];
	while(infile.getline(buf, 1024))
	{
	    // Written this way, it will process %S in the helpfiles
	    // Instead of notice(s_GameServ, u, "%s", buf);
	    	notice(s_GameServ, u, buf);
	}

	// Minor recursion
	aClient *user = find(u);
	if (user && isAdmin(user))
	    display_help(u, "admin_commands");
    }
    else
    {
	char *filename;

	for (unsigned int x = 0; x < strlen(file); x++)
	    file[x] = tolower(file[x]);

	filename = new char[strlen(file) + 12];
	sprintf(filename, "helpfiles/%s", file);
	infile.open(filename);
	delete [] filename;
	if (infile.fail())
	{
	    notice(s_GameServ, u, "No help for %s", file);
	    return;
	}
	buf = new char[1024];
	while(infile.getline(buf, 1024))
	{
	    // Written this way, it will process %S in the helpfiles
	    // Instead of notice(s_GameServ, u, "%s", buf);
	    notice(s_GameServ, u, buf);
	}
    }
    infile.close();
    delete [] buf;
}

void do_admin(char *u)
{
    aClient *user;
    char *pass = strtok(NULL, " ");

    if (!(user = find(u)))
    {
	cout << "Error: aClient not found: " << u << endl;
	notice(s_GameServ, u, "Error: aClient not found. Contact %S admin.");
	return;
    }
    if (!pass)
    {
	notice(s_GameServ, u, "SYNTAX: ADMIN password");
	return;
    }

    if (isAdmin(user))
    {
	notice(s_GameServ, u, "You already have administrator privledges.");
	return;
    }
    else if (strcmp(pass, adminpass) == 0)
    {
	notice(s_GameServ, u, "Password accepted. You now have administrator privledges.");
	setAdmin(user);
    }
    else
    {
	notice(s_GameServ, u, "Invalid password. Remember: case sensitive");
	return;
    }
}

bool load_monsters()
{
    ifstream infile;
    infile.open("monsters.dat");

    char *buf;

    if (infile.fail())
    {
	cout << "Error opening monsters.dat" << endl;
	return false;
    }
    init_monsters();
    buf = new char[2048];

  for (int l = 0; l < REALLEVELS; l++)
  {
    for (int m = 0; m < MONSTERS;)
    {
	infile.getline(buf, 2048);
	if (buf[0] == '\n' || buf[0] == '\0' || buf[0] == '#')
	    continue;
	else
	{
	    strcpy(monsters[l][m]->name, strtok(buf, "~"));
	    strcpy(monsters[l][m]->weapon, strtok(NULL, "~"));
	    monsters[l][m]->strength = stringtoint(strtok(NULL, "~"));
	    monsters[l][m]->gold = stringtoint(strtok(NULL, "~"));
	    monsters[l][m]->exp = stringtoint(strtok(NULL, "~"));
	    monsters[l][m]->maxhp = stringtoint(strtok(NULL, "~"));
	    strcpy(monsters[l][m]->death, strtok(NULL, ""));
	    m++;
	}
    }
  }
    delete [] buf;
return true;
}
